@@ -1,4 +1,5 @@
 import sys
+import Numeric
 from numarray import *
 from numarray import strings
 from tables import *
@@ -19,9 +20,17 @@ a = array(4, Int16)
 hdfarray = fileh.createArray(root, 'array_s', a, "Scalar signed short array")
 
 # Create a 3-d array of floats
-a = arange(64, type=Float64, shape=(2,4,8))
+#a = arange(64, type=Float64, shape=(2,4,8))
+a = arange(120, type=Float64, shape=(20,3,2))
 # Save it on the HDF5 file
 hdfarray = fileh.createArray(root, 'array_f', a, "3-D float array")
+# Crea a Numeric array
+a = Numeric.arange(120, typecode="d")
+#Numeric.reshape(a,(20,3,2))
+a.shape=(20,3,2)
+print "a.shape-->", a.shape
+# Save it on the HDF5 file
+hdfarray = fileh.createArray(root, 'array_f_Numeric', a, "3-D float array, Numeric")
 
 # Close the file
 fileh.close()
