@@ -3,6 +3,7 @@
 #include "tables.h"
 #include "H5Zlzo.h"
 #include "H5Zucl.h"
+#include "utils.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -20,11 +21,11 @@
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: F. Alted. October 21, 2002
+ * Programmer: F. Altet. October 21, 2002
  *
  * Date: March 19, 2001
  *
- * Comments: Modified by F. Alted. November 07, 2003
+ * Comments: Modified by F. Altet. November 07, 2003
  *
  * Modifications: This is the same routine as H5LTmake_dataset, but I've 
  *                added a TITLE attribute for array, as well as 
@@ -36,6 +37,7 @@
 
 herr_t H5ARRAYmake( hid_t loc_id, 
 		    const char *dset_name,
+		    const char *klass,
 		    const char *title,
 		    const char *flavor,
 		    const char *obversion,    /* The Array VERSION number */
@@ -43,7 +45,7 @@ herr_t H5ARRAYmake( hid_t loc_id,
 		    const hsize_t *dims,
 		    int   extdim,
 		    hid_t type_id,
-		    hsize_t max_tuples,
+		    hsize_t *dims_chunk,
 		    void  *fill_data,
 		    int   compress,
 		    char  *complib,
@@ -54,7 +56,6 @@ herr_t H5ARRAYmake( hid_t loc_id,
 
  hid_t   dataset_id, space_id, datatype;  
  hsize_t *maxdims = NULL;
- hsize_t *dims_chunk = NULL;
  hid_t   plist_id = 0;
  unsigned int cd_values[3];
  int     chunked = 0;
@@ -67,18 +68,14 @@ herr_t H5ARRAYmake( hid_t loc_id,
 
  if (chunked) {
    maxdims = malloc(rank*sizeof(hsize_t));
-   dims_chunk = malloc(rank*sizeof(hsize_t));
 
    for(i=0;i<rank;i++) {
+/*      printf("dims_chunk[%d]: %d\n", i, dims_chunk[i]); */
      if (i == extdim) {
        maxdims[i] = H5S_UNLIMITED;
-       dims_chunk[i] = max_tuples;
-       /* If set to 1, makes the chunk incompressible. */
-/*        dims_chunk[i] = 1; */
      }
      else {
        maxdims[i] = dims[i];
-       dims_chunk[i] = dims[i];
      }
    }
  }
@@ -188,17 +185,13 @@ herr_t H5ARRAYmake( hid_t loc_id,
  */
     
  /* Attach the CLASS attribute */
- if (extdim >= 0) {
-   if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", "EARRAY" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", klass ) < 0 )
      goto out;
+ if (extdim >= 0) {
    /* Attach the EXTDIM attribute in case of enlargeable arrays */
    if ( H5LTset_attribute_int( loc_id, dset_name, "EXTDIM", &extdim, 1 ) < 0 )
      goto out;
  }
- else {
-   if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", "ARRAY" ) < 0 )
-     goto out;
- }
    
  /* Attach the FLAVOR attribute */
  if ( H5LTset_attribute_string( loc_id, dset_name, "FLAVOR", flavor ) < 0 )
@@ -215,8 +208,6 @@ herr_t H5ARRAYmake( hid_t loc_id,
  /* Release resources */
  if (maxdims)
    free(maxdims);
- if (dims_chunk)
-   free(dims_chunk);
 
  return dataset_id;
 
@@ -240,7 +231,7 @@ out:
  * Return: Success: 0, Failure: -1
  *
  * Programmers: 
- *  Francesc Alted
+ *  Francesc Altet
  *
  * Date: October 30, 2003
  *
@@ -266,8 +257,8 @@ herr_t H5ARRAYappend_records( hid_t loc_id,
  hid_t    type_id;
  hid_t    space_id;
  hid_t    mem_space_id;
- hsize_t  *dims;
- hssize_t *start;
+ hsize_t  *dims = NULL;  	/* Shut up the compiler */
+ hssize_t *start = NULL;        /* Shut up the compiler */
  int      i;
 
  /* Open the dataset. */
@@ -283,7 +274,6 @@ herr_t H5ARRAYappend_records( hid_t loc_id,
  start = malloc(rank*sizeof(hsize_t));
  for(i=0;i<rank;i++) {
    dims[i] = dims_orig[i];
-/*    printf("dims_new[%d]: %d\n",i, (int)dims_new[i]); */
    start[i] = 0;
  }
  dims[extdim] += dims_new[extdim];
@@ -297,11 +287,6 @@ herr_t H5ARRAYappend_records( hid_t loc_id,
  if ( (mem_space_id = H5Screate_simple( rank, dims_new, NULL )) < 0 )
   return -1;
 
-/*  for (i=0; i<rank;i++) { */
-/*    printf("dims[%d] = %d\n", i, (int)dims[i]); */
-/*    printf("start[%d] = %d\n", i, (int)start[i]); */
-/*    printf("dims_new[%d] = %d\n", i, (int)dims_new[i]); */
-/*  } */
  /* Get the file data space */
  if ( (space_id = H5Dget_space( dataset_id )) < 0 )
   return -1;
@@ -325,7 +310,7 @@ herr_t H5ARRAYappend_records( hid_t loc_id,
  
  /* Release the datatype. */
  if ( H5Tclose( type_id ) < 0 )
-  return -1;
+  goto out;
 
  /* End access to the dataset */
  if ( H5Dclose( dataset_id ) < 0 )
@@ -339,10 +324,183 @@ return 0;
 
 out:
  H5Dclose( dataset_id );
+ if (start) free(start);
+ if (dims) free(dims);
  return -1;
 
 }
 
+/*-------------------------------------------------------------------------
+ * Function: H5ARRAYwrite_records
+ *
+ * Purpose: Write records to an array
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmers: 
+ *  Francesc Altet
+ *
+ * Date: October 26, 2004
+ *
+ * Comments: Uses memory offsets
+ *
+ * Modifications: 
+ *
+ *
+ *-------------------------------------------------------------------------
+ */
+
+
+herr_t H5ARRAYwrite_records( hid_t loc_id, 
+			     const char *dset_name,
+			     const int rank,
+			     hsize_t *start,
+			     hsize_t *step,
+			     hsize_t *count,
+			     const void *data )  
+{
+
+ hid_t    dataset_id;
+ hid_t    type_id;
+ hid_t    space_id;
+ hid_t    mem_space_id;
+
+ /* Open the dataset. */
+ if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
+  goto out;
+
+ /* Get the datatype */
+ if ( (type_id = H5Dget_type( dataset_id )) < 0 )
+  goto out;
+
+ /* Create a simple memory data space */
+ if ( (mem_space_id = H5Screate_simple( rank, count, NULL )) < 0 )
+  return -1;
+
+ /* Get the file data space */
+ if ( (space_id = H5Dget_space( dataset_id )) < 0 )
+  return -1;
+
+ /* Define a hyperslab in the dataset */
+ if ( H5Sselect_hyperslab( space_id, H5S_SELECT_SET, start, step, count, NULL) < 0 )
+   goto out;
+
+ if ( H5Dwrite( dataset_id, type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
+     goto out;
+
+ /* Terminate access to the dataspace */
+ if ( H5Sclose( mem_space_id ) < 0 )
+  goto out;
+ 
+ if ( H5Sclose( space_id ) < 0 )
+  goto out;
+ 
+ /* Release the datatype. */
+ if ( H5Tclose( type_id ) < 0 )
+  goto out;
+
+ /* End access to the dataset */
+ if ( H5Dclose( dataset_id ) < 0 )
+  goto out;
+
+return 0;
+
+out:
+ H5Dclose( dataset_id );
+ return -1;
+
+}
+
+
+/*-------------------------------------------------------------------------
+ * Function: H5ARRAYtruncate
+ *
+ * Purpose: Truncate the EArray to at most size rows
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmers: 
+ *  Francesc Altet
+ *
+ * Date: November 19, 2004
+ *
+ * Comments:
+ *
+ * Modifications: 
+ *
+ *
+ *-------------------------------------------------------------------------
+ */
+
+
+herr_t H5ARRAYtruncate( hid_t loc_id, 
+			const char *dset_name,
+			const int extdim,
+			const hsize_t size)
+{
+ hid_t    dataset_id;
+ hid_t    space_id;
+ hsize_t  *dims = NULL;
+ int      rank;
+
+/*  printf("size (1): %d\n", size); */
+ /* Open the dataset. */
+ if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
+  goto out;
+
+  /* Get the dataspace handle */
+ if ( (space_id = H5Dget_space( dataset_id )) < 0 )
+  goto out;
+ 
+ /* Get the rank */
+ if ( (rank = H5Sget_simple_extent_ndims(space_id)) < 0 )
+   goto out;
+
+ if (rank) {  			/* Array case */
+   /* Book some memory for the selections */
+   dims = (hsize_t *)malloc(rank*sizeof(hsize_t));
+
+   /* Get dataset dimensionality */
+   if ( H5Sget_simple_extent_dims(space_id, dims, NULL) < 0 )
+     goto out;
+
+   if ( size >= dims[extdim] ) {
+     printf("Asking for truncate to more rows that the available ones!.\n");
+     goto out;
+   }
+
+   /* Truncate the EArray */
+   dims[extdim] = size;
+   if ( H5Dset_extent( dataset_id, dims ) < 0 )
+/*    if ( H5Dextend( dataset_id, dims ) < 0 ) */
+     goto out;
+
+   /* Release resources */
+   free(dims);
+ }
+ else {
+     printf("An scalar Array cannot be truncated!.\n");
+     goto out;
+ }
+
+ /* Free resources */
+ if ( H5Sclose( space_id ) < 0 )
+   return -1;
+
+ /* End access to the dataset */
+ if ( H5Dclose( dataset_id ) < 0 )
+  return -1;
+
+ return size;
+
+out:
+ H5Dclose( dataset_id );
+ if (dims) free(dims);
+ return -1;
+
+}  
+
+
 /*-------------------------------------------------------------------------
  * Function: H5ARRAYread
  *
@@ -350,7 +508,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, falted@pytables.org
  *
  * Date: October 22, 2002
  *
@@ -486,7 +644,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted, falted@pytables.org
+ * Programmer: Francesc Altet, falted@pytables.org
  *
  * Date: December 16, 2003
  *
@@ -540,6 +698,8 @@ herr_t H5ARRAYreadSlice( hid_t loc_id,
    for(i=0;i<rank;i++) {
      count[i] = ((stop[i] - start[i] - 1) / step[i]) + 1;
 /*      printf("dims[%d]: %d\n", i, (int)dims[i]); */
+/*      printf("start[%d]: %d\n", i, (int)start[i]); */
+/*      printf("stop[%d]: %d\n", i, (int)stop[i]); */
 /*      printf("offset[%d]: %d\n", i, (int)offset[i]); */
 /*      printf("count[%d]: %d\n", i, (int)count[i]); */
 /*      printf("stride[%d]: %d\n", i, (int)stride[i]); */
@@ -599,6 +759,152 @@ out:
 }
   
 
+/*   The next represents a try to implement getCoords for != operator */
+/*   but it turned out to be too difficult, well, at least to me :( */
+/*   2004-06-22 */
+/*-------------------------------------------------------------------------
+ * Function: H5ARRAYreadIndex
+ *
+ * Purpose: Reads a slice of array from disk for indexing purposes.
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmer: Francesc Altet, falted@pytables.org
+ *
+ * Date: June 21, 2004
+ *
+ *-------------------------------------------------------------------------
+ */
+
+herr_t H5ARRAYreadIndex( hid_t loc_id, 
+			 const char *dset_name,
+			 int notequal,
+			 hsize_t *start,
+			 hsize_t *stop,
+			 hsize_t *step,
+			 void *data )
+{
+ hid_t    dataset_id;  
+ hid_t    space_id;
+ hid_t    mem_space_id;
+ hid_t    type_id;
+ hsize_t  *dims = NULL;
+ hsize_t  *count = NULL;
+ hsize_t  *count2 = NULL;
+ hsize_t  *offset2 = NULL;
+ hsize_t  *stride = (hsize_t *)step;
+ hssize_t *offset = (hssize_t *)start;
+ int      rank;
+ int      i;
+
+ /* Open the dataset. */
+ if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
+  return -1;
+ 
+ /* Get the datatype */
+ if ( (type_id = H5Dget_type(dataset_id)) < 0 )
+     return -1;
+ 
+  /* Get the dataspace handle */
+ if ( (space_id = H5Dget_space( dataset_id )) < 0 )
+  goto out;
+ 
+ /* Get the rank */
+ if ( (rank = H5Sget_simple_extent_ndims(space_id)) < 0 )
+   goto out;
+
+ if (rank) {  			/* Array case */
+
+   /* Book some memory for the selections */
+   dims = (hsize_t *)malloc(rank*sizeof(hsize_t));
+   count = (hsize_t *)malloc(rank*sizeof(hsize_t));
+   count2 = (hsize_t *)malloc(rank*sizeof(hsize_t));
+   offset2 = (hsize_t *)malloc(rank*sizeof(hsize_t));
+
+   /* Get dataset dimensionality */
+   if ( H5Sget_simple_extent_dims( space_id, dims, NULL) < 0 )
+     goto out;
+
+   for(i=0;i<rank;i++) {
+     count[i] = ((stop[i] - start[i] - 1) / step[i]) + 1;
+/*      printf("dims[%d]: %d\n", i, (int)dims[i]); */
+/*      printf("start[%d]: %d\n", i, (int)start[i]); */
+/*      printf("stop[%d]: %d\n", i, (int)stop[i]); */
+/*      printf("offset[%d]: %d\n", i, (int)offset[i]); */
+/*      printf("count[%d]: %d\n", i, (int)count[i]); */
+/*      printf("stride[%d]: %d\n", i, (int)stride[i]); */
+     if ( stop[i] > dims[i] ) {
+       printf("Asking for a range of rows exceeding the available ones!.\n");
+       goto out;
+     }
+   }
+
+   /* Define a hyperslab in the dataset of the size of the records */
+   if ( H5Sselect_hyperslab( space_id, H5S_SELECT_SET, offset, stride, count, NULL) < 0 )
+     goto out;
+
+   /* If we want the complementary, do a NOTA against all the row */
+   if (notequal) {
+     offset2[0] = offset[0]; count2[0] = count[0];
+     offset2[1] = 0; count2[1] = dims[1]; /* All the row */
+     count[0] = 1; count[1] = dims[1] - count[1]; /* For memory dataspace */
+     printf("dims[%d]: %d\n", i, (int)dims[i]);
+     printf("offset2[%d]: %d\n", i, (int)offset2[i]);
+     printf("count2[%d]: %d\n", i, (int)count2[i]);
+     printf("count[%d]: %d\n", i, (int)count[i]);
+     if ( H5Sselect_hyperslab( space_id, H5S_SELECT_NOTA, offset2, stride, count2, NULL) < 0 )
+       goto out;
+   }
+
+   /* Create a memory dataspace handle */
+   if ( (mem_space_id = H5Screate_simple( rank, count, NULL )) < 0 )
+     goto out;
+
+   /* Read */
+   if ( H5Dread( dataset_id, type_id, mem_space_id, space_id, H5P_DEFAULT, data ) < 0 )
+     goto out;
+
+   /* Release resources */
+   free(dims);
+   free(count);
+   free(offset2);
+   free(count2);
+
+   /* Terminate access to the memory dataspace */
+   if ( H5Sclose( mem_space_id ) < 0 )
+     goto out;
+ }
+ else {  			/* Scalar case */
+
+   /* Read all the dataset */
+   if (H5Dread(dataset_id, type_id, H5S_ALL, H5S_ALL, H5P_DEFAULT, data) < 0)
+     goto out;
+ }
+
+   /* Terminate access to the dataspace */
+ if ( H5Sclose( space_id ) < 0 )
+  goto out;
+
+ /* End access to the dataset and release resources used by it. */
+ if ( H5Dclose( dataset_id ) )
+  return -1;
+
+ /* Close the vlen type */
+ if ( H5Tclose(type_id))
+   return -1;
+
+ return 0;
+
+out:
+ H5Dclose( dataset_id );
+ if (dims) free(dims);
+ if (count) free(count);
+ return -1;
+
+}
+  
+
+
 /*-------------------------------------------------------------------------
  * Function: H5ARRAYget_ndims
  *
@@ -606,14 +912,14 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Francesc Alted
+ * Programmer: Francesc Altet
  *
  * Date: October 22, 2002
  *
  *-------------------------------------------------------------------------
  */
 /* Addition: Now, this routine can deal with both array and
-   atomic datatypes. F. Alted  2003-01-29 */
+   atomic datatypes. F. Altet  2003-01-29 */
 
 herr_t H5ARRAYget_ndims( hid_t loc_id, 
 			 const char *dset_name,
@@ -657,7 +963,6 @@ herr_t H5ARRAYget_ndims( hid_t loc_id,
  
   }
 
-
  /* Terminate access to the datatype */
  if ( H5Tclose( type_id ) < 0 )
   goto out;
@@ -674,6 +979,58 @@ out:
 
 }
 
+
+/*-------------------------------------------------------------------------
+ * Function: H5ARRAYget_chunksize
+ *
+ * Purpose: Gets the chunksize along the extensible dimension of an earray.
+ *
+ * Return: Success: chunksize, Failure: -1
+ *
+ * Programmer: Francesc Altet
+ *
+ * Date: May 20, 2004
+ *
+ *-------------------------------------------------------------------------
+ */
+
+herr_t H5ARRAYget_chunksize( hid_t loc_id, 
+			     const char *dset_name,
+			     int rank,
+			     hsize_t *dims_chunk)
+{
+  hid_t       dataset_id;  
+  hid_t       plist_id;
+
+  /* Open the dataset. */
+  if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
+    return -1;
+
+  /* Get creation properties list */
+  if ( (plist_id = H5Dget_create_plist( dataset_id )) < 0 )
+    goto out;
+
+  /* Get the chunksize for all dimensions */
+  if (H5Pget_chunk(plist_id, rank, dims_chunk ) < 0 )
+    goto out;
+
+ /* Terminate access to the datatype */
+ if ( H5Pclose( plist_id ) < 0 )
+  goto out;
+
+ /* End access to the dataset */
+ if ( H5Dclose( dataset_id ) )
+  return -1;
+
+ return 0;
+
+out:
+ if (dims_chunk) free(dims_chunk);
+ H5Dclose( dataset_id );
+ return -1;
+
+}
+
 /* Modified version of H5LTget_dataset_info present on HDF_HL
  * I had to add the capability to get the sign of
  * the array type.
@@ -681,18 +1038,18 @@ out:
 /* Addition: Now, this routine can deal with both array and
    atomic datatypes. 2003-01-29 */
 
-herr_t H5ARRAYget_info( hid_t loc_id, 
-			const char *dset_name,
-			hsize_t *dims,
-			hid_t *super_type_id,
-			H5T_class_t *super_class_id,
-			char *byteorder)
+hid_t H5ARRAYget_info( hid_t loc_id, 
+		       const char *dset_name,
+		       hsize_t *dims,
+		       hid_t *super_type_id,
+		       H5T_class_t *super_class_id,
+		       char *byteorder)
 {
   hid_t       dataset_id;  
   hid_t       space_id; 
   H5T_class_t class_arr_id;
   H5T_order_t order;
-  hid_t       type_id; 
+  hid_t       type_id;
 
   /* Open the dataset. */
   if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
@@ -743,8 +1100,8 @@ herr_t H5ARRAYget_info( hid_t loc_id,
   /* Get the byteorder */
   /* Only class integer and float can be byteordered */
   if ( (*super_class_id == H5T_INTEGER) || (*super_class_id == H5T_FLOAT)
-       || (*super_class_id == H5T_BITFIELD) ) {
-    order = H5Tget_order( *super_type_id );
+       || (*super_class_id == H5T_BITFIELD) || (*super_class_id == H5T_COMPOUND)) {
+    order = get_order( *super_type_id );
     if (order == H5T_ORDER_LE) 
       strcpy(byteorder, "little");
     else if (order == H5T_ORDER_BE)
@@ -762,7 +1119,7 @@ herr_t H5ARRAYget_info( hid_t loc_id,
   if ( H5Dclose( dataset_id ) )
     return -1;
 
-  return 0;
+  return dataset_id;
 
 out:
  H5Tclose( type_id );
