@@ -19,7 +19,7 @@ herr_t H5TBOopen_read( hid_t *dataset_id,
 		       hsize_t nfields,
 		       char **field_names,
 		       size_t type_size,
-		       const size_t *field_offset );
+		       size_t *field_offset );
 
 herr_t H5TBOread_records( hid_t *dataset_id, hid_t *space_id,
 			  hid_t *mem_type_id, hsize_t start,
@@ -60,3 +60,12 @@ herr_t H5TBOclose_append(hid_t *dataset_id,
 			 hsize_t ntotal_records,
 			 const char *dset_name,
 			 hid_t parent_id);
+
+herr_t H5TBOwrite_records( hid_t loc_id, 
+			   const char *dset_name,
+			   hsize_t start,
+			   hsize_t nrecords,
+			   hsize_t step,
+			   size_t type_size,
+			   const size_t *field_offset,
+			   const void *data );
