@@ -2,7 +2,7 @@
 #include "tables.h"
 #include "H5Zlzo.h"
 #include "H5Zucl.h"
-
+#include "utils.h"
 #include <string.h>
 #include <stdlib.h>
 
@@ -377,6 +377,93 @@ out:
 }
 
 
+/*-------------------------------------------------------------------------
+ * Function: H5ARRAYmodify_records
+ *
+ * Purpose: Modify records of an array
+ *
+ * Return: Success: 0, Failure: -1
+ *
+ * Programmers: 
+ *  Francesc Alted
+ *
+ * Date: October 28, 2004
+ *
+ * Comments: Uses memory offsets
+ *
+ * Modifications: 
+ *
+ *
+ *-------------------------------------------------------------------------
+ */
+
+herr_t H5VLARRAYmodify_records( hid_t loc_id, 
+				const char *dset_name,
+				hsize_t nrow,
+				int nobjects,
+				const void *data )  
+{
+
+ hid_t    dataset_id;
+ hid_t    type_id;
+ hid_t    space_id;
+ hid_t    mem_space_id;
+ hssize_t start[1];
+ hsize_t  dims_new[1] = {1};	/* Only a record on each update */
+ hvl_t    wdata;   /* Information to write */
+
+ /* Initialize VL data to write */
+ wdata.p=(void *)data;
+ wdata.len=nobjects;
+
+ /* Open the dataset. */
+ if ( (dataset_id = H5Dopen( loc_id, dset_name )) < 0 )
+  goto out;
+
+ /* Get the datatype */
+ if ( (type_id = H5Dget_type( dataset_id )) < 0 )
+  goto out;
+
+ /* Create a simple memory data space */
+ if ( (mem_space_id = H5Screate_simple( 1, dims_new, NULL )) < 0 )
+  return -1;
+
+ /* Get the file data space */
+ if ( (space_id = H5Dget_space( dataset_id )) < 0 )
+  return -1;
+
+ /* Define a hyperslab in the dataset */
+ start[0] = nrow;
+ if ( H5Sselect_hyperslab( space_id, H5S_SELECT_SET, start, NULL, dims_new, NULL) < 0 )
+   goto out;
+
+ if ( H5Dwrite( dataset_id, type_id, mem_space_id, space_id, H5P_DEFAULT, &wdata ) < 0 )
+     goto out;
+
+ /* Terminate access to the dataspace */
+ if ( H5Sclose( space_id ) < 0 )
+  goto out;
+ 
+ if ( H5Sclose( mem_space_id ) < 0 )
+  goto out;
+ 
+ /* Release the datatype. */
+ if ( H5Tclose( type_id ) < 0 )
+  return -1;
+
+ /* End access to the dataset */
+ if ( H5Dclose( dataset_id ) < 0 )
+  goto out;
+
+return 1;
+
+out:
+ H5Dclose( dataset_id );
+ return -1;
+
+}
+
+
 /*-------------------------------------------------------------------------
  * Function: H5VLARRAYread
  *
@@ -575,12 +662,12 @@ out:
  *-------------------------------------------------------------------------
  */
 
-herr_t H5VLARRAYget_info( hid_t   loc_id, 
-			  char    *dset_name,
-			  hsize_t *nrecords,
-			  hsize_t *base_dims,
-			  hid_t   *base_type_id,
-			  char    *base_byteorder )
+hid_t H5VLARRAYget_info( hid_t   loc_id, 
+			 char    *dset_name,
+			 hsize_t *nrecords,
+			 hsize_t *base_dims,
+			 hid_t   *base_type_id,
+			 char    *base_byteorder )
 {
   hid_t       dataset_id;  
   hid_t       type_id;
@@ -636,9 +723,9 @@ herr_t H5VLARRAYget_info( hid_t   loc_id,
 
   /* Get the byteorder */
   /* Only class integer and float can be byteordered */
-  if ( (base_class_id == H5T_INTEGER) || (base_class_id == H5T_FLOAT)
-       || (base_class_id == H5T_BITFIELD) ) {
-    order = H5Tget_order( *base_type_id );
+  if ((base_class_id == H5T_INTEGER) || (base_class_id == H5T_FLOAT)
+      || (base_class_id == H5T_BITFIELD) || (base_class_id == H5T_COMPOUND)) {
+    order = get_order( *base_type_id );
     if (order == H5T_ORDER_LE) 
       strcpy(base_byteorder, "little");
     else if (order == H5T_ORDER_BE)
@@ -660,7 +747,7 @@ herr_t H5VLARRAYget_info( hid_t   loc_id,
   if ( H5Dclose( dataset_id ) )
     return -1;
 
-  return 0;
+  return dataset_id;
 
 out:
  H5Dclose( dataset_id );
