@@ -5,7 +5,7 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/Array.py,v $
-#       $Id: Array.py,v 1.63 2004/03/09 07:48:56 falted Exp $
+#       $Id: Array.py,v 1.79.2.1 2004/11/11 10:47:22 falted Exp $
 #
 ########################################################################
 
@@ -27,11 +27,12 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.63 $"
+__version__ = "$Revision: 1.79.2.1 $"
 
 # default version for ARRAY objects
 #obversion = "1.0"    # initial version
-obversion = "2.0"    # Added an optional EXTDIM attribute
+#obversion = "2.0"    # Added an optional EXTDIM attribute
+obversion = "2.1"    # Added support for complex datatypes
 
 
 import types, warnings, sys
@@ -104,7 +105,16 @@ class Array(Leaf, hdf5Extension.Array, object):
         global obversion
 
         self._v_version = obversion
-        naarr, self.flavor = self._convertIntoNA(self.object)
+        try:
+            naarr, self.flavor = self._convertIntoNA(self.object)
+        except:
+            # Problems converting data. Close this node.
+            #print "Problems converting input object:", str(self.object)
+            self.close(flush=0)
+            (type, value, traceback) = sys.exc_info()
+            # Re-raise the exception
+            raise type, value
+
         if naarr.shape:
             self._v_expectedrows = naarr.shape[0]
         else:
@@ -134,7 +144,14 @@ class Array(Leaf, hdf5Extension.Array, object):
         else:
             self.nrows = 1    # Scalar case
         self.itemsize = naarr.itemsize()
-        self.type = self._createArray(naarr, self._v_new_title)
+        try:
+            self.type = self._createArray(naarr, self._v_new_title)
+        except:
+            # Problems creating the Array on disk. Close this node
+            self.close(flush=0)
+            (type, value, traceback) = sys.exc_info()
+            # Re-raise the exception
+            raise type, value
 
     def _convertIntoNA(self, object):
         "Convert a generic object into a numarray object"
@@ -193,7 +210,7 @@ class Array(Leaf, hdf5Extension.Array, object):
                 # If still doesn't, issues an error
                 except:
                     raise ValueError, \
-"""The object '%s' can't be converted to a numerical or character array.
+"""The object '%s' can't be converted into a numerical or character array.
   Sorry, but this object is not supported.""" % (arr)
             if isinstance(arr, types.TupleType):
                 flavor = "Tuple"
@@ -222,11 +239,12 @@ class Array(Leaf, hdf5Extension.Array, object):
 
     def _open(self):
         """Get the metadata info for an array in file."""
-        (self.type, self.shape, self.itemsize, self.byteorder) = \
-                        self._openArray()
+        (self.type, self.shape, self.itemsize, self.byteorder,
+         self._v_chunksize) = self._openArray()
+        
         # Compute the rowsize for each element
         self.rowsize = self.itemsize
-        for i in range(len(self.shape)):
+        for i in xrange(len(self.shape)):
             self.rowsize *= self.shape[i]
         # Assign a value to nrows in case we are a non-enlargeable object
         if self.shape:
@@ -235,20 +253,13 @@ class Array(Leaf, hdf5Extension.Array, object):
             self.nrows = 1   # Scalar case
         # Compute the optimal chunksize
         (self._v_maxTuples, self._v_chunksize) = \
-                   calcBufferSize(self.rowsize, self.nrows,
-                                  self.filters.complevel)
+                            calcBufferSize(self.rowsize, self.nrows,
+                                           self.filters.complevel)
+            
 
     def iterrows(self, start=None, stop=None, step=None):
-        """Iterator over all the rows or a range"""
-
-        return self.__call__(start, stop, step)
-
-    def __call__(self, start=None, stop=None, step=None):
         """Iterate over all the rows or a range.
         
-        It returns the same iterator than
-        Table.iterrows(start, stop, step).
-        It is, therefore, a shorter way to call it.
         """
 
         try:
@@ -287,7 +298,6 @@ class Array(Leaf, hdf5Extension.Array, object):
             del self._init
             raise StopIteration        # end of iteration
         else:
-            #print "start, stop, step:", self._start, self._stop, self._step
             # Read a chunk of rows
             if self._row+1 >= self._v_maxTuples or self._row < 0:
                 self._stopb = self._startb+self._step*self._v_maxTuples
@@ -315,25 +325,28 @@ class Array(Leaf, hdf5Extension.Array, object):
             self._row += 1
             self.nrow += self._step
             self._nrowsread += self._step
-            if self.listarr.shape:
+            # Fixes bug #968132
+            #if self.listarr.shape:
+            if self.shape:
                 return self.listarr[self._row]
             else:
                 return self.listarr    # Scalar case
 
     def __getitem__(self, keys):
-        """Returns an Array row or slice.
+        """Returns an Array element, row or extended slice.
 
         It takes different actions depending on the type of the "key"
         parameter:
 
-        If "key"is an integer, the corresponding row is returned. If
+        If "key" is an integer, the corresponding row is returned. If
         "key" is a slice, the row slice determined by key is returned.
 
         """
 
-        if self.shape == ():
-            # Scalar case
-            raise IndexError, "You cannot read scalar Arrays through indexing. Try using the read() method better."
+#	The following restriction is unnecessary: it should work fine for keys == () ...
+#        if self.shape == ():
+#            # Scalar case
+#            raise IndexError, "You cannot read scalar Arrays through indexing. Try using the read() method better."
 
         maxlen = len(self.shape)
         shape = (maxlen,)
@@ -354,12 +367,20 @@ class Array(Leaf, hdf5Extension.Array, object):
                       self._v_pathname
             if isinstance(key, types.EllipsisType):
                 ellipsis = 1
-                for diml in range(dim, len(self.shape) - (nkeys - dim) + 1):
+                for diml in xrange(dim, len(self.shape) - (nkeys - dim) + 1):
                     startl[dim] = 0
                     stopl[dim] = self.shape[diml]
                     stepl[dim] = 1
                     dim += 1
             elif isinstance(key, types.IntType):
+                # Index out of range protection
+                if key >= self.shape[dim]:
+                    raise IndexError, "Index out of range"
+                if key < 0:
+                    # To support negative values (Fixes bug #968149)
+                    key += self.shape[dim]
+#                 start, stop, step = processRangeRead(self.shape[dim],
+#                                                      key, key+1, 1)
                 start, stop, step = processRange(self.shape[dim],
                                                  key, key+1, 1)
                 stop_None[dim] = 1
@@ -377,7 +398,7 @@ class Array(Leaf, hdf5Extension.Array, object):
             
         # Complete the other dimensions, if needed
         if dim < len(self.shape):
-            for diml in range(dim, len(self.shape)):
+            for diml in xrange(dim, len(self.shape)):
                 startl[dim] = 0
                 stopl[dim] = self.shape[diml]
                 stepl[dim] = 1
@@ -385,6 +406,112 @@ class Array(Leaf, hdf5Extension.Array, object):
 
         return self._readSlice(startl, stopl, stepl, stop_None)
 
+
+    def __setitem__(self, key, value):
+        """Sets an Array element, row or extended slice.
+
+        It takes different actions depending on the type of the "key"
+        parameter:
+
+        If "key" is an integer, the corresponding row is assigned to
+        value.
+        
+        If "key" is a slice, the row slice determined by it is
+        assigned to "value". If needed, this "value" is broadcasted to
+        fit in the desired range. If the slice to be updated exceeds
+        the actual shape of the array, only the values in the existing
+        range are updated, i.e. the index error will be silently
+        ignored. If "value" is a multidimensional object, then its
+        shape must be compatible with the slice specified in "key",
+        otherwhise, a ValueError will be issued.
+
+        """
+
+        if self.shape == ():
+            # Scalar case
+            raise IndexError, "You cannot read scalar Arrays through indexing. Try using the read() method better."
+
+        maxlen = len(self.shape)
+        shape = (maxlen,)
+        startl = numarray.array(None, shape=shape, type=numarray.Int64)
+        stopl = numarray.array(None, shape=shape, type=numarray.Int64)
+        stepl = numarray.array(None, shape=shape, type=numarray.Int64)
+        stop_None = numarray.zeros(shape=shape, type=numarray.Int64)
+        if not isinstance(key, types.TupleType):
+            keys = (key,)
+        else:
+            keys = key
+        nkeys = len(keys)
+        dim = 0
+        # Here is some problem when dealing with [...,...] params
+        # but this is a bit weird way to pass parameters anyway
+        for key in keys:
+            ellipsis = 0  # Sentinel
+            if dim >= maxlen:
+                raise IndexError, "Too many indices for object '%s'" % \
+                      self._v_pathname
+            if isinstance(key, types.EllipsisType):
+                ellipsis = 1
+                for diml in xrange(dim, len(self.shape) - (nkeys - dim) + 1):
+                    
+                    startl[dim] = 0
+                    stopl[dim] = self.shape[diml]
+                    stepl[dim] = 1
+                    dim += 1
+            elif isinstance(key, types.IntType):
+                # Index out of range protection
+                if key >= self.shape[dim]:
+                    raise IndexError, "Index out of range"
+                if key < 0:
+                    # To support negative values (Fixes bug #968149)
+                    key += self.shape[dim]
+                start, stop, step = processRange(self.shape[dim],
+                                                 key, key+1, 1)
+                stop_None[dim] = 1
+            elif isinstance(key, types.SliceType):
+                start, stop, step = processRange(self.shape[dim],
+                                                 key.start, key.stop, key.step)
+            else:
+                raise ValueError, "Non-valid index or slice: %s" % \
+                      key
+            if not ellipsis:
+                startl[dim] = start
+                stopl[dim] = stop
+                stepl[dim] = step
+                dim += 1
+            
+        # Complete the other dimensions, if needed
+        if dim < len(self.shape):
+            for diml in xrange(dim, len(self.shape)):
+                startl[dim] = 0
+                stopl[dim] = self.shape[diml]
+                stepl[dim] = 1
+                dim += 1
+
+        # Create an array compliant with the specified slice
+        countl = ((stopl - startl - 1) / stepl) + 1
+        if str(self.type) == "CharType":
+            if shape <> []:
+                narr = strings.array(None, itemsize=self.itemsize,
+                                     shape=countl)
+            else:
+                narr = strings.array([""], itemsize=self.itemsize,
+                                     shape=countl)
+        else:
+            narr = numarray.array(None, shape=countl, type=self.type)
+
+        # Assign the value to it
+        try:
+            narr[:] = value
+        except:
+            (type, value2, traceback) = sys.exc_info()
+            raise ValueError, \
+"value parameter '%s' cannot be converted into an array object compliant with earray: \n'%r'\nThe error was: <%s>" % \
+        (value, self, value2)
+
+        if narr.size():
+            self._modify(startl, stepl, countl, narr)
+
     # Accessor for the _readArray method in superclass
     def _readSlice(self, startl, stopl, stepl, stop_None):
 
@@ -394,37 +521,59 @@ class Array(Leaf, hdf5Extension.Array, object):
             extdim = self.extdim
 
         shape = []
-        for dim in range(len(self.shape)):
+        for dim in xrange(len(self.shape)):
             new_dim = ((stopl[dim] - startl[dim] - 1) / stepl[dim]) + 1
             if not (new_dim == 1 and stop_None[dim]):
                 # Append dimension
                 shape.append(new_dim)
+
+        # The next solution isn't appropriate as a scalar array is
+        # meant as a way to return a Python value
+#         if shape == []:
+#             # In case of scalar elements, make them equivalent to 1dim 
+#             # This is much better for dealing with single-element objects
+#             shape = [1]  
             
         if repr(self.type) == "CharType":
-            arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+            # Workaround for numarray bug #997997
+            if shape <> []:
+                arr = strings.array(None, itemsize=self.itemsize, shape=shape)
+            else:
+                arr = strings.array([""], itemsize=self.itemsize, shape=shape)
         else:
-            arr = numarray.zeros(type=self.type, shape=shape)
+            #arr = numarray.zeros(type=self.type, shape=shape)
+            # This is slightly faster (~3%) than zeros()
+            arr = numarray.array(None,type=self.type, shape=shape)
             # Set the same byteorder than on-disk
             arr._byteorder = self.byteorder
+
         # Protection against reading empty arrays
         zerodim = 0
-        for i in range(len(shape)):
+        for i in xrange(len(shape)):
             if shape[i] == 0:
                 zerodim = 1
 
         if not zerodim:
             # Arrays that have non-zero dimensionality
-            self._g_readSlice(startl, stopl, stepl, arr._data)
-            pass
+            self._g_readSlice(startl, stopl, stepl, arr)
 
         if hasattr(self, "_v_convert") and self._v_convert == 0:
             return arr
-            
-        if self.flavor not in ["NumArray", "CharArray"]:
-            return self._convToFlavor(arr)
+
+        if self.flavor in ["NumArray", "CharArray"]:
+            if arr.shape == ():  # Scalar case
+                return arr[()] 
+            else:             # No conversion needed
+                return arr
+        # Fixes #968131
+        elif arr.shape == ():  # Scalar case
+            return arr[()]  # return the value.
+        # The next solution isn't appropriate as a scalar array is
+        # meant as a way to return a Python value
+#         elif arr.shape == (1,):  # Scalar case
+#             return arr[0]  # return the value.
         else:
-            # No conversion needed
-            return arr
+            return self._convToFlavor(arr)
         
     def _convToFlavor(self, arr):
         "Convert the numarray parameter to the correct flavor"
@@ -446,14 +595,15 @@ class Array(Leaf, hdf5Extension.Array, object):
                     else:
                         # This works for rank-0 arrays
                         # (but is slower for big arrays)
-                        arr=Numeric.array(arr, typecode=arr.typecode())
+                        arr=Numeric.array(arr[()], typecode=arr.typecode())
                         
             else:
                 # Warn the user
                 warnings.warn( \
 """The object on-disk has Numeric flavor, but Numeric is not installed locally. Returning a numarray object instead!.""")
         elif self.flavor == "Tuple":
-            arr = tuple(arr.tolist())
+            #arr = tuple(arr.tolist())
+            arr = self.totuple(arr)
         elif self.flavor == "List":
             arr = arr.tolist()
         elif self.flavor == "Int":
@@ -464,6 +614,13 @@ class Array(Leaf, hdf5Extension.Array, object):
             arr = arr.tostring()
 
         return arr
+    
+    def totuple(self, arr):
+        """Returns array as a (nested) tuple of elements."""
+        if len(arr._shape) == 1:
+            return tuple([ x for x in arr ])
+        else:
+            return tuple([ self.totuple(ni) for ni in arr ])
         
     # Accessor for the _readArray method in superclass
     def read(self, start=None, stop=None, step=None):
@@ -483,14 +640,12 @@ class Array(Leaf, hdf5Extension.Array, object):
         if repr(self.type) == "CharType":
             arr = strings.array(None, itemsize=self.itemsize, shape=shape)
         else:
-            # keyword buffer deprecated
-            #arr = numarray.array(buffer=None, type=self.type, shape=shape)
             arr = numarray.array(None, type=self.type, shape=shape)
             # Set the same byteorder than on-disk
             arr._byteorder = self.byteorder
         # Protection against reading empty arrays
         zerodim = 0
-        for i in range(len(shape)):
+        for i in xrange(len(shape)):
             if shape[i] == 0:
                 zerodim = 1
 
@@ -499,7 +654,11 @@ class Array(Leaf, hdf5Extension.Array, object):
             self._readArray(start, stop, step, arr._data)
             
         if self.flavor in ["NumArray", "CharArray"]:
-            return arr  # No further conversions needed
+            # No conversion needed
+            return arr
+        # Fixes #968131
+        elif arr.shape == ():  # Scalar case
+            return arr[()]  # return the value. Yes, this is a weird syntax :(
         else:
             return self._convToFlavor(arr)
         
@@ -507,7 +666,10 @@ class Array(Leaf, hdf5Extension.Array, object):
         "Private part of Leaf.copy() for each kind of leaf"
         # Get the slice of the array
         # (non-buffered version)
-        arr = self[start:stop:step]
+	if self.shape:
+            arr = self[start:stop:step]
+	else:
+	    arr = self[()]
         # Build the new Array object
         object = Array(arr, title=title)
         setattr(group, name, object)
