@@ -4,7 +4,7 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/File.py,v $
-#       $Id: File.py,v 1.82 2004/02/25 16:08:59 falted Exp $
+#       $Id: File.py,v 1.91.2.3 2004/11/16 18:52:21 falted Exp $
 #
 ########################################################################
 
@@ -34,11 +34,13 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.82 $"
+__version__ = "$Revision: 1.91.2.3 $"
 #format_version = "1.0" # Initial format
 #format_version = "1.1" # Changes in ucl compression
-format_version = "1.2"  # Support for enlargeable arrays and VLA's
+#format_version = "1.2"  # Support for enlargeable arrays and VLA's
                         # 1.2 was introduced in pytables 0.8
+format_version = "1.3"  # Support for indexes in Tables
+                        # 1.3 was introduced in pytables 0.9
 compatible_formats = [] # Old format versions we can read
                         # Empty means that we support all the old formats
 
@@ -186,9 +188,9 @@ def openFile(filename, mode="r", title="", trMap={}, rootUEP="/",
             
             elif not hdf5Extension.isPyTablesFile(path):
                 warnings.warn( \
-"""'%s' does exist, is an HDF5 file, but has not a PyTables format.
-  Trying to guess what's here from HDF5 metadata. I can't promise you getting
-  the correct objects, but I will do my best!."""  % path, UserWarning)
+"""\n'%s' does exist, is an HDF5 file, but has not a PyTables format. Trying to guess what's there using HDF5 metadata. I can't promise you getting the correct objects, but I will do my best!.""" % \
+path, UserWarning)
+  
                 isPTFile = 0
                     
     elif (mode == "w"):
@@ -247,7 +249,15 @@ class File(hdf5Extension.File, object):
         renameNode(where, newname [, name])
         getAttrNode(self, where, attrname [, name])
         setAttrNode(self, where, attrname, attrname [, name])
+        delAttrNode(self, where, attrname [, name])
+        copyAttrs(self, where, name, dstNode)
+        copyChildren(self, whereSrc, whereDst [, recursive] [, filters]
+                     [, copyuserattrs] [, start] [, stop ] [, step]
+                     [, overwrite])
+        copyFile(self, dstFilename [, title]
+                 [, filters] [, copyuserattrs] [, overwrite])
         walkGroups([where])
+        walkNodes([where] [, classname])
         flush()
         close()
 
@@ -401,15 +411,18 @@ class File(hdf5Extension.File, object):
                 self.format_version = "unknown"
                           
             # Get the title for the rootGroup group
-            rootGroup.__dict__["_v_title"] = attrsRoot.TITLE
+            if hasattr(attrsRoot, "TITLE"):
+                rootGroup.__dict__["_v_title"] = attrsRoot.TITLE
+            else:
+                rootGroup.__dict__["_v_title"] = ""
             # Get the title for the file
             #self.title = hdf5Extension.read_f_attr(self._v_objectID, 'TITLE')
             self.title = rootGroup._v_title
             # Get the filters for the file
-            filters = attrsRoot.FILTERS
-            if filters is None:
-                filters = Filters()
-            self.filters = filters
+            if hasattr(attrsRoot, "FILTERS"):
+                self.filters = attrsRoot.FILTERS
+            else:
+                self.filters = Filters()
                       
             # Get all the groups recursively
             rootGroup._g_openFile()
@@ -567,14 +580,13 @@ class File(hdf5Extension.File, object):
             information about the desired I/O filters to be applied
             during the life of this object.
 
-        expectedrows -- In the case of enlargeable arrays this
-            represents an user estimate about the number of row
-            elements that will be added to the growable dimension in
-            the EArray object. If not provided, the default value is
-            1000 rows. If you plan to create both much smaller or much
-            bigger EArrays try providing a guess; this will optimize
-            the HDF5 B-Tree creation and management process time and
-            the amount of memory used.
+        expectedrows -- Represents an user estimate about the number
+            of row elements that will be added to the growable
+            dimension in the EArray object. If not provided, the
+            default value is 1000 rows. If you plan to create both
+            much smaller or much bigger EArrays try providing a guess;
+            this will optimize the HDF5 B-Tree creation and management
+            process time and the amount of memory used.
 
             """
 
@@ -584,7 +596,6 @@ class File(hdf5Extension.File, object):
         setattr(group, name, Object)
         return Object
 
-
     def createVLArray(self, where, name, atom=None, title="",
                       filters=None, expectedsizeinMB=1.0,
                       compress=None, complib=None):
@@ -749,7 +760,6 @@ class File(hdf5Extension.File, object):
         
         """
 
-        # Get the node to be renamed
         object = self.getNode(where, name=name)
         if isinstance(object, Group):
             return object._f_getAttr(attrname)
@@ -768,21 +778,37 @@ class File(hdf5Extension.File, object):
         
         """
 
-        # Get the node to be renamed
         object = self.getNode(where, name=name)
         if isinstance(object, Group):
             object._f_setAttr(attrname, attrvalue)
         else:
             object.setAttr(attrname, attrvalue)
         
-    def copyAttrs(self, where, name="", dstNode=None):
-        """Copy the attributes from node "where"."name" to "dstNode".
+    def delAttrNode(self, where, attrname, name = ""):
+        """Delete the attribute "attrname" of node "where"."name".
 
         "where" can be a path string or Group instance. If "where"
         doesn't exists or has not a child called "name", a LookupError
         error is raised. If "name" is a null string (""), or not
         supplied, this method assumes to find the object in "where".
-        "dstNode" is the destination and can be whether a path string
+        "attrname" is the name of the attribute to delete.
+        
+        """
+
+        object = self.getNode(where, name=name)
+        if isinstance(object, Group):
+            return object._f_delAttr(attrname)
+        else:
+            return object.delAttr(attrname)
+            
+    def copyAttrs(self, where, name="", dstNode=None):
+        """Copy the attributes from node "where"."name" to "dstNode".
+
+        "where" can be a path string or Group instance. If "where"
+        doesn't exist or has not a child called "name", a LookupError
+        error is raised. If "name" is a null string (""), or not
+        supplied, this method assumes to find the object in "where".
+        "dstNode" is the destination and can be either a path string
         or a Node object.
         
         """
@@ -884,11 +910,15 @@ class File(hdf5Extension.File, object):
     def __iter__(self, where="/", classname=""):
         """Iterate over the nodes in the object tree."""
 
-        return self._iterTree(where, classname)
+        return self.walkNodes(where, classname)
 
-    def _iterTree(self, where="/", classname=""):
-        """Iterate over the nodes in the object tree."""
+    def walkNodes(self, where="/", classname=""):
+        """Iterate over the nodes in the object tree.
+        If "where" supplied, the iteration starts from this group.
+        If "classname" is supplied, only instances of this class are
+        returned.
         
+        """        
         if classname == "Group":
             for group in self.walkGroups(where):
                 yield group
@@ -902,17 +932,6 @@ class File(hdf5Extension.File, object):
                 for leaf in self.listNodes(group, classname):
                     yield leaf
                 
-    def __call__(self, where="/", classname=""):
-        """Iterate over the nodes in the object tree.
-
-        If "where" supplied, the iteration starts from this group.
-        If "classname" is supplied, only instances of this class are
-        returned.
-
-        """
-
-        return self.__iter__(where, classname)
-
     def walkGroups(self, where = "/"):
         """Returns the list of Groups (not Leaves) hanging from "where".
 
@@ -928,16 +947,16 @@ class File(hdf5Extension.File, object):
 
                     
     def flush(self):
-        
         """Flush all the objects on all the HDF5 objects tree."""
 
         for group in self.walkGroups(self.root):
             for leaf in self.listNodes(group, classname = 'Leaf'):
                 leaf.flush()
-            
+
+        # Flush the cache to disk
+        self._flushFile(0)  # 0 means local scope, 1 global (virtual) scope
                 
-    def close(self):
-        
+    def close(self):        
         """Close all the objects in HDF5 file and close the file."""
 
         # If the file is already closed, return immediately
@@ -949,31 +968,17 @@ class File(hdf5Extension.File, object):
         for group in self.walkGroups(self.root):
             for leaf in self.listNodes(group, classname = 'Leaf'):
                 leaf.close()
-            #print "File: Closing Group ==>", group
-            #if group._v_pathname <> "/":
-            #    group._f_close()
             group._f_close()
             
-        # Delete the root object (this should recursively delete the
-        # object tree)
-        #         del self.root    # not necessary
-
-        # Pass Mr proper
-#         for group in listgroups:
-#             group.__dict__.clear()
-#             group._v_children.clear()
-#         self.groups.clear()
-
         # Close the file
         self._closeFile()
                     
-        # Set the flag to indicate that the file is closed
-        self.isopen = 0
-
         # After the objects are disconnected, destroy the
         # object dictionary using the brute force ;-)
         # This should help to the garbage collector
-        #self.__dict__.clear()
+        self.__dict__.clear()
+        # Set the flag to indicate that the file is closed
+        self.isopen = 0
 
         return
 
@@ -982,15 +987,12 @@ class File(hdf5Extension.File, object):
         
         # Print all the nodes (Group and Leaf objects) on object tree
         date = time.asctime(time.localtime(os.stat(self.filename)[8]))
-        astring = "Filename: " + repr(self.filename) + ' '
-        if self.title <> "unknown":
-            astring += "Title: "+repr(self.title)
-        astring += "; Last modif.: " + repr(date)
-        astring += '; rootUEP=' + repr(self.rootUEP)
-        astring += '; filters=' + repr(self.filters)
-
-        if self.format_version <> "unknown":
-            astring += "; Format version: " + self.format_version + '\n'
+        astring =  self.filename + ' (File) ' + repr(self.title) + '\n'
+#         astring += 'rootUEP :=' + repr(self.rootUEP) + '; '
+#         astring += 'format_version := ' + self.format_version + '\n'
+#         astring += 'filters :=' + repr(self.filters) + '\n'
+        astring += 'Last modif.: ' + repr(date) + '\n'
+        astring += 'Object Tree: \n'
 
         for group in self.walkGroups("/"):
             astring += str(group) + '\n'
