@@ -5,7 +5,7 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/Group.py,v $
-#       $Id: Group.py,v 1.72 2004/02/25 16:08:59 falted Exp $
+#       $Id: Group.py,v 1.82.2.2 2004/11/11 10:47:23 falted Exp $
 #
 ########################################################################
 
@@ -33,11 +33,11 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.72 $"
+__version__ = "$Revision: 1.82.2.2 $"
 
+# Recommended values for maximum number of groups and maximum depth in tree
+# However, these limits are somewhat arbitraries and can be increased
 MAX_DEPTH_IN_TREE = 2048
-# Note: the next constant has to be syncronized with the
-# MAX_CHILDS_IN_GROUP constant in util.h!
 MAX_CHILDS_IN_GROUP = 4096
 
 from __future__ import generators
@@ -47,6 +47,7 @@ import hdf5Extension
 from Table import Table
 from Array import Array
 from EArray import EArray
+from IndexArray import IndexArray
 from VLArray import VLArray
 from Leaf import Filters
 from UnImplemented import UnImplemented
@@ -79,6 +80,7 @@ class Group(hdf5Extension.Group, object):
     
         _f_listNodes(classname)
         _f_walkGroups()
+        _f_walkNodes(classname, recursive)
         __delattr__(name)
         __getattr__(name)
         __setattr__(name, object)
@@ -87,6 +89,7 @@ class Group(hdf5Extension.Group, object):
         _f_remove(recursive=0)
         _f_getAttr(attrname)
         _f_setAttr(attrname, attrvalue)
+        _f_delAttr(attrname)
         _f_copyChildren(where, recursive=0, filters=None, copyuserattrs=1,
                         start=0, stop=None, step=1, overwrite=0)
         _f_close()
@@ -107,6 +110,7 @@ class Group(hdf5Extension.Group, object):
         _v_leaves -- Dictionary with object leaves
         _v_children -- Dictionary with object children (groups or leaves)
         _v_nchildren -- Number of children (groups or leaves) of this object 
+        _v_indices -- List with indices hanging from this group
         _v_attrs -- The associated AttributeSet instance
         _v_filters -- The associated Filters instance
 
@@ -127,14 +131,22 @@ class Group(hdf5Extension.Group, object):
         self.__dict__["_v_leaves"] = {}
         self.__dict__["_v_children"] = {}
         self.__dict__["_v_nchildren"] = 0
+        self.__dict__["_v_indices"] = []
         return
     
     def __iter__(self, classname=None, recursive=0):
         """Iterate over the children on self"""
 
-        return self._f_iterGroup(classname, recursive)
+        return self._f_walkNodes(classname, recursive)
 
-    def _f_iterGroup(self, classname=None, recursive=0):
+    def _f_walkNodes(self, classname=None, recursive=0):
+        """Iterate over the nodes of self
+
+        If "classname" is supplied, only instances of this class
+        are returned. If "recursive" is false, only children
+        hanging immediately after the group are returned. If
+        true, a recursion over all the groups hanging from it is
+        performed. """
 
         if not recursive:
             # Non-recursive algorithm
@@ -150,17 +162,6 @@ class Group(hdf5Extension.Group, object):
                     for leaf in group._f_listNodes(classname):
                         yield leaf
                 
-    def __call__(self, classname=None, recursive=0):
-        """Iterate over the children on self
-
-	      If "classname" is supplied, only instances of this class
-	      are returned. If "recursive" is false, only children
-	      hanging immediately after the group are returned. If
-	      true, a recursion over all the groups hanging from it is
-	      performed. """
-        
-        return self.__iter__(classname, recursive)
-
     # This iterative version of _g_openFile is due to John Nielsen
     def _g_openFile(self):
         """Recusively reads an HDF5 file and generates a tree object.
@@ -174,13 +175,23 @@ class Group(hdf5Extension.Group, object):
             (groups, leaves)=self._g_listGroup(pgroupId, locId,
                                                objgroup._v_hdf5name)
             for name in groups:
-                new_objgroup = Group(new = 0)
-                new_objgroup._g_putObjectInTree(name, objgroup)
-                stack.append(new_objgroup)
+                # Index groups will not be included in the object tree
+                #if objgroup._g_getGChildAttr(name, "CLASS") != "INDEX":
+                classname = objgroup._g_getGChildAttr(name, "CLASS")
+                if classname not in ["INDEX", "IARRAY"]: # Delete the "IARRAY"
+                    new_objgroup = Group(new = 0)
+                    new_objgroup._g_putObjectInTree(name, objgroup)
+                    stack.append(new_objgroup)
+                else:
+                    # and their names will be append to a list
+                    if classname == "INDEX":
+                        # only INDEX names will be appended
+                        self._v_indices.append(name)
             for name in leaves:
                 objleaf=objgroup._g_getLeaf(name)
                 if objleaf <> None:
                     # Try if object can be loaded
+                    #objleaf._g_putObjectInTree(name, objgroup)
                     try:
                         objleaf._g_putObjectInTree(name, objgroup)
                     except:
@@ -219,6 +230,8 @@ self._g_join(name), UserWarning)
             return Table()
         elif class_ == "ARRAY":
             return Array()
+        elif class_ == "IMAGE":
+            return Array()
         elif class_ == "EARRAY":
             return EArray()
         elif class_ == "VLARRAY":
@@ -328,7 +341,10 @@ self._g_join(name), UserWarning)
 
     # Define _v_title as a property
     def _f_get_title (self):
-        return self._v_attrs.TITLE
+        if hasattr(self._v_attrs, "TITLE"):
+            return self._v_attrs.TITLE
+        else:
+            return ""
     
     def _f_set_title (self, title):
         self._v_attrs.TITLE = title
@@ -422,7 +438,8 @@ self._g_join(name), UserWarning)
         "classname" parameter is supplied, it will only return
         instances of this class (or subclasses of it). The supported
         classes in "classname" are 'Group', 'Leaf', 'Table', 'Array',
-        'EArray', 'VLArray' and 'UnImplemented'.
+        'EArray', 'VLArray' and 'UnImplemented'. 'IndexArray' objects
+        are not listed by default.
 
         """
         if not classname:
@@ -443,6 +460,7 @@ self._g_join(name), UserWarning)
         elif (classname == 'Table' or
               classname == 'Array' or
               classname == 'EArray' or
+              #classname == 'IndexArray' or
               classname == 'VLArray' or
               classname == 'UnImplemented'):
             listobjects = []
@@ -454,7 +472,7 @@ self._g_join(name), UserWarning)
             return listobjects
         else:
             raise ValueError, \
-""""classname" can only take 'Group', 'Leaf', 'Table', 'Array', 'EArray', 'VLArray', 'UnImplemented' values"""
+""""classname" can only take 'Group', 'Leaf', 'Table', 'Array', 'EArray', 'IndexArray', 'VLArray', 'UnImplemented' values"""
 
     def _f_walkGroups(self):
         """Iterate over the Groups (not Leaves) hanging from self.
@@ -550,7 +568,7 @@ self._g_join(name), UserWarning)
 """'%s' group is exceeding the recommended maximum number of children (%d).
  Be ready to see PyTables asking for *lots* of memory and possibly slow I/O.
 """ % (self._v_pathname, MAX_CHILDS_IN_GROUP), UserWarning)
-            
+
         # Put value object with name "name" in object tree
         if name not in self._v_children:
             value._g_putObjectInTree(name, self)
@@ -559,6 +577,10 @@ self._g_join(name), UserWarning)
                   "'%s' group already has a child named '%s' in file '%s'" % \
                   (self._v_pathname, name, self._v_rootgroup._v_filename)
 
+    def _f_flush(self):
+        """ Flush this Group """
+        self._g_flushGroup()
+
     def _f_close(self):
         """Close this HDF5 group"""
         self._g_closeGroup()
@@ -580,6 +602,9 @@ self._g_join(name), UserWarning)
         # Detach the AttributeSet instance
         self._v_attrs._f_close()
         del self.__dict__["_v_attrs"]
+        # Delete the filters instance
+        if self.__dict__.has_key("_v_filters"):
+            del self.__dict__["_v_filters"]
 
     def _f_getAttr(self, attrname):
         """Get a group attribute as a string"""
@@ -591,6 +616,11 @@ self._g_join(name), UserWarning)
 
         setattr(self._v_attrs, attrname, attrvalue)
 
+    def _f_delAttr(self, attrname):
+        """Delete an group attribute as a string"""
+
+        delattr(self._v_attrs, attrname)
+
     def _f_rename(self, newname):
         """Rename a group"""
 
