@@ -5,14 +5,12 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/IsDescription.py,v $
-#       $Id: IsDescription.py,v 1.30 2004/02/25 16:08:59 falted Exp $
+#       $Id: IsDescription.py,v 1.35 2004/09/24 11:58:13 falted Exp $
 #
 ########################################################################
 
 """Classes and metaclasses for defining user data columns for Table objects.
 
-See the metaIsDescription for a deep explanation on how exactly this works.
-
 Classes:
 
     metaIsDescription
@@ -26,7 +24,7 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.30 $"
+__version__ = "$Revision: 1.35 $"
 
 
 import warnings
@@ -44,6 +42,7 @@ tostructfmt = {NA.Int8:'b', NA.UInt8:'B',
                NA.Int32:'i', NA.UInt32:'I',
                NA.Int64:'q', NA.UInt64:'Q',
                NA.Float32:'f', NA.Float64:'d',
+               NA.Complex32:'F', NA.Complex64:'D', # Added top support complex
                NA.Bool:'c', records.CharType:'s', 
                }
 
@@ -53,14 +52,18 @@ fromstructfmt = {'b':NA.Int8, 'B':NA.UInt8,
                  'i':NA.Int32, 'I':NA.UInt32,
                  'q':NA.Int64, 'Q':NA.UInt64,
                  'f':NA.Float32, 'd':NA.Float64,
+                 'F':NA.Complex32, 'D':NA.Complex64,# Added top support complex
                  'c':NA.Bool, 's':records.CharType,
-              }
+                 }
 
+#class BaseCol:
 class Col:
-    """ Define a column """
-    def __init__(self, dtype="Float64", shape=1, dflt=None, pos=None):
+    """ Define a general column that supports all numarray data types"""
+    def __init__(self, dtype="Float64", shape=1, dflt=None, pos=None,
+                 indexed=0):
 
         self.pos = pos
+        self.indexed = indexed
 
         assert shape != None and shape != 0, \
                "None or zero-valued shapes are not supported '%s'" % `shape`
@@ -85,6 +88,9 @@ class Col:
             self.type = NA.typeDict[dtype]
             self.recarrtype = records.revfmt[self.type]
             self.itemsize = self.type.bytes
+            if self.indexed and (dtype == NA.Complex32 or
+                                 dtype == NA.Complex64):
+                raise TypeError, "'%s' do not support indexation" % repr(dtype)
         elif dtype == "CharType" or isinstance(dtype, records.Char):
             # Special case for Strings
             self.type = records.CharType
@@ -124,20 +130,21 @@ class Col:
               ", shape=" +  str(shape) + \
               ", dflt=" + str(self.dflt) + \
               ", pos=" + str(self.pos) + \
+              ", indexed=" + str(self.indexed) + \
               ")"
         return out
 
-    # Just a test
-    def __close(self):
-        self.__dict__.clear()
+#     # Just a test
+#     def __close(self):
+#         self.__dict__.clear()
         
-
 class BoolCol(Col):
     """ Define a string column """
     
-    def __init__(self, dflt=0, shape=1, pos=None):
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
 
         self.pos = pos
+        self.indexed = indexed
 
         assert shape != None and shape != 0, \
                "None or zero-valued shapes are not supported '%s'" % `shape`
@@ -163,9 +170,11 @@ class BoolCol(Col):
 class StringCol(Col):
     """ Define a string column """
     
-    def __init__(self, length=None, dflt=None, shape=1, pos=None):
+    def __init__(self, length=None, dflt=None, shape=1, pos=None,
+                 indexed=0):
 
         self.pos = pos
+        self.indexed = indexed
 
         assert isinstance(dflt, types.StringTypes) or dflt == None, \
                "Invalid default value: '%s'" % dflt
@@ -201,9 +210,11 @@ class StringCol(Col):
     
 class IntCol(Col):
     """ Define an integer column """
-    def __init__(self, dflt=0, shape=1, itemsize=4, sign=1, pos=None):
+    def __init__(self, dflt=0, shape=1, itemsize=4, sign=1, pos=None,
+                 indexed=0):
 
         self.pos = pos
+        self.indexed = indexed
 
         assert shape != None and shape != 0, \
                "None or zero-valued shapes are not supported '%s':" % `shape`
@@ -250,49 +261,59 @@ class IntCol(Col):
 
 class Int8Col(IntCol):
     "Description class for a signed integer of 8 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt, itemsize=1, shape=shape, sign=1, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt, itemsize=1, shape=shape, sign=1,
+                        pos=pos, indexed=indexed)
         
 class UInt8Col(IntCol):
     "Description class for an unsigned integer of 8 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=1, shape=shape, sign=0, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=1, shape=shape, sign=0,
+                        pos=pos, indexed=indexed)
         
 class Int16Col(IntCol):
     "Description class for a signed integer of 16 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=2, shape=shape, sign=1, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=2, shape=shape, sign=1,
+                        pos=pos, indexed=indexed)
         
 class UInt16Col(IntCol):
     "Description class for an unsigned integer of 16 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=2, shape=shape, sign=0, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=2, shape=shape, sign=0,
+                        pos=pos, indexed=indexed)
         
 class Int32Col(IntCol):
     "Description class for a signed integer of 32 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=4, shape=shape, sign=1, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=4, shape=shape, sign=1,
+                        pos=pos, indexed=indexed)
         
 class UInt32Col(IntCol):
     "Description class for an unsigned integer of 32 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=4, shape=shape, sign=0, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=4, shape=shape, sign=0,
+                        pos=pos, indexed=indexed)
         
 class Int64Col(IntCol):
     "Description class for a signed integer of 64 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=8, shape=shape, sign=1, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=8, shape=shape, sign=1,
+                        pos=pos, indexed=indexed)
         
 class UInt64Col(IntCol):
     "Description class for an unsigned integer of 64 bits "
-    def __init__(self, dflt=0, shape=1, pos=None):
-        IntCol.__init__(self, dflt , itemsize=8, shape=shape, sign=0, pos=pos)
+    def __init__(self, dflt=0, shape=1, pos=None, indexed=0):
+        IntCol.__init__(self, dflt , itemsize=8, shape=shape, sign=0,
+                        pos=pos, indexed=indexed)
         
 class FloatCol(Col):
     """ Define a float column """
-    def __init__(self, dflt=0.0, shape=1, itemsize=8, pos=None):
+    def __init__(self, dflt=0.0, shape=1, itemsize=8, pos=None,
+                 indexed=0):
 
         self.pos = pos
+        self.indexed = indexed
 
         assert shape != None and shape != 0, \
                "None or zero-valued shapes are not supported '%s'" % `shape`
@@ -323,15 +344,60 @@ class FloatCol(Col):
 
 class Float32Col(FloatCol):
     "Description class for a floating point of 32 bits "
-    def __init__(self, dflt=0.0, shape=1, pos=None):
-        FloatCol.__init__(self, dflt , shape=shape, itemsize=4, pos=pos)
+    def __init__(self, dflt=0.0, shape=1, pos=None, indexed=0):
+        FloatCol.__init__(self, dflt , shape=shape, itemsize=4,
+                          pos=pos, indexed=indexed)
         
 class Float64Col(FloatCol):
     "Description class for a floating point of 64 bits "
-    def __init__(self, dflt=0.0, shape=1, pos=None):
-        FloatCol.__init__(self, dflt , shape=shape, itemsize=8, pos=pos)
+    def __init__(self, dflt=0.0, shape=1, pos=None, indexed=0):
+        FloatCol.__init__(self, dflt , shape=shape, itemsize=8,
+                          pos=pos, indexed=indexed)
         
-    
+class ComplexCol(Col):
+    """ Define a complex column """
+    def __init__(self, dflt=(0.0+0.0j), shape=1, itemsize=16, pos=None):
+
+        self.pos = pos
+        self.indexed = 0  # This attribuet is needed
+        assert shape != None and shape != 0, \
+               "None or zero-valued shapes are not supported '%s'" % `shape`
+
+        assert itemsize in [8, 16], \
+               "Copmplex itemsizes different from 8 and 16 are not supported"
+        
+        if type(shape) in [types.IntType, types.LongType]:
+            # To prevent confusions between 2 and (2,): the meaning is the same
+            if shape == 1:
+                self.shape = shape
+            else:
+                self.shape = (shape,)
+        elif type(shape) in [types.ListType, types.TupleType]:
+            self.shape = tuple(shape)
+        else: raise ValueError, "Illegal shape %s" % `shape`
+
+        self.dflt = dflt
+
+        self.itemsize = itemsize
+        if itemsize == 8:
+            self.type = NA.typeDict["Complex32"]
+        elif itemsize == 16:
+            self.type = NA.typeDict["Complex64"]
+                
+        self.recarrtype = records.revfmt[self.type]
+        self.rectype = tostructfmt[self.type]
+
+class Complex32Col(ComplexCol):
+    "Description class for a complex of simple precision "
+    def __init__(self, dflt=(0.0+0.0j), shape=1, pos=None):
+        ComplexCol.__init__(self, dflt, shape=shape, itemsize=8, pos=pos)
+        
+class Complex64Col(ComplexCol):
+    "Description class for a complex of double precision "
+    def __init__(self, dflt=(0.0+0.0j), shape=1, pos=None):
+        ComplexCol.__init__(self, dflt , shape=shape, itemsize=16, pos=pos)
+
+
 class Description(object):
     "Regular class to keep table description metadata"
 
@@ -358,8 +424,8 @@ class Description(object):
             keys.sort()
             
         recarrfmt = []
-        if "__check_validity" in keys:
-            check_validity = classdict["__check_validity"]
+        if "__check_validity__" in keys:
+            check_validity = classdict["__check_validity__"]
         else:
             check_validity = 1   # Default value for name validity check
         for k in keys:
@@ -369,9 +435,6 @@ class Description(object):
                     warnings.warn("Can't set attr %r in coldescr-class %r" % (
                         k, classname))
                 else:
-                    # Beware, in this case, we don't allow fields with
-                    # prefix "_v_". This is reserved to pass special
-                    # variables to the new class.
                     #print "Special variable!:", k
                     newdict[k] = classdict[k]
             else:
@@ -464,6 +527,8 @@ class Description(object):
             dfltvalue = int(0)
         elif datatype in ('f', 'd'):
             dfltvalue = float(0)
+        elif datatype in ('F', 'D'):
+            dfltvalue = complex(0)
         elif datatype in ('c',):
 #             dfltvalue = str(" ")
             dfltvalue = int(0)
@@ -513,7 +578,9 @@ class metaIsDescription(type):
         newdict = {"columns":{},
                    }
         for k in classdict.keys():
-            if not (k.startswith('__') or k.startswith('_v_')):
+            #if not (k.startswith('__') or k.startswith('_v_')):
+            # We let pass _v_ variables to configure class behaviour
+            if not (k.startswith('__')):
                 newdict["columns"][k] = classdict[k]
 
         # Return a new class with the "columns" attribute filled
