@@ -5,7 +5,7 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/Leaf.py,v $
-#       $Id: Leaf.py,v 1.46 2004/02/25 16:08:59 falted Exp $
+#       $Id: Leaf.py,v 1.55.2.1 2004/11/10 13:55:48 falted Exp $
 #
 ########################################################################
 
@@ -28,7 +28,7 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.46 $"
+__version__ = "$Revision: 1.55.2.1 $"
 
 import types, warnings
 from utils import checkNameValidity, calcBufferSize, processRangeRead
@@ -36,7 +36,7 @@ from AttributeSet import AttributeSet
 import Group
 import hdf5Extension
 
-class Filters:
+class Filters(object):
     """Container for filter properties
 
     Instance variables:
@@ -92,27 +92,32 @@ class Filters:
             self.complib = "zlib"   # Should always exists
 
     def __repr__(self):
+        """The string reprsentation choosed for this object.
+        """
         filters = "Filters("
-        if self.complevel:
-            filters += "complevel=%s" % (self.complevel)
-            filters += ", complib='%s'" % (self.complib)
-            if self.shuffle:
-                filters += ", shuffle=%s" % (self.shuffle)
-            if self.fletcher32:
-                filters += ", "
-        if self.fletcher32:
-            filters += "fletcher32=%s" % (self.fletcher32)
+#         if self.complevel:
+#             filters += "complevel=%s" % (self.complevel)
+#             filters += ", complib='%s'" % (self.complib)
+#             if self.shuffle:
+#                 filters += ", shuffle=%s" % (self.shuffle)
+#             if self.fletcher32:
+#                 filters += ", "
+#         if self.fletcher32:
+#             filters += "fletcher32=%s" % (self.fletcher32)
+        filters += "complevel=%s" % (self.complevel)
+        filters += ", complib='%s'" % (self.complib)
+        filters += ", shuffle=%s" % (self.shuffle)
+        filters += ", fletcher32=%s" % (self.fletcher32)
         filters += ")"
         return filters
     
     def __str__(self):
-        """The string reprsentation choosed for this object is its pathname
-        in the HDF5 object tree.
+        """The string reprsentation choosed for this object.
         """
         
         return repr(self)
 
-class Leaf:
+class Leaf(object):
     """A class to place common functionality of all Leaf objects.
 
     A Leaf object is all the nodes that can hang directly from a
@@ -126,10 +131,11 @@ class Leaf:
 
         close()
         flush()
-        getAttr(attrname)
         rename(newname)
         remove()
+        getAttr(attrname)
         setAttr(attrname, attrvalue)
+        delAttr(attrname)
 
     Instance variables:
 
@@ -141,6 +147,10 @@ class Leaf:
         byteorder -- the byteorder of the leaf
         filters -- information for active filters
         attrs -- The associated AttributeSet instance
+        _v_parent -- the parent Group instance
+        _v_rootgroup -- always point to the root group object
+        _v_file -- the associated File object
+        _v_depth -- the depth level in tree for this leaf
 
     """
 
@@ -162,12 +172,13 @@ class Leaf:
             # Set the filters instance variable
             self.filters = self._g_setFilters(self._v_new_filters)
             self._create()
-            # Write the Filters object to an attribute
-            # This will not be necessary for now, as retrieving
-            # the filters using hdf5Extension._getFilters is safer and
-            # faster. Also, cPickling the filters attribute is
-            # very slow (it is as much as twice slower than the normal
-            # overhead for creating a Table, for example).
+                        
+            # Write the Filters object to an attribute This will not
+            # be necessary for now, as retrieving the filters using
+            # hdf5Extension._getFilters is safer and faster. Also,
+            # cPickling the filters attribute is very slow (it is as
+            # much as twice slower than the normal overhead for
+            # creating a Table, for example).
             #self.attrs._g_setAttr("FILTERS", self.filters)
         else:
             self.filters = self._g_getFilters()
@@ -345,7 +356,11 @@ class Leaf:
         # Check that the name does not exist under this group
         if group._v_children.has_key(name):
             if overwrite:
-                # Delete the destination object
+                if group == self._v_parent and name == self.name:
+		    # Trying to overwrite itself!. Silently give up...
+		    # This fixes bug #973370
+		    return (self, 0)  # 0 bytes copied
+		# Delete the destination object
                 dstNode = getattr(group, name)
                 if dstNode.__class__.__name__ == "Group":
                     dstNode._f_remove(recursive=1)
@@ -376,7 +391,7 @@ class Leaf:
         "Remove a leaf"
         parent = self._v_parent
         parent._g_deleteLeaf(self._v_name)
-        self.close()
+        self.close(flush=0)
 
     def rename(self, newname):
         """Rename a leaf"""
@@ -403,13 +418,20 @@ class Leaf:
 
         setattr(self.attrs, attrname, attrvalue)
 
+    def delAttr(self, attrname):
+        """Delete a leaf attribute as a string"""
+
+        delattr(self.attrs, attrname)
+
     def flush(self):
         """Save whatever remaining data in buffer"""
-        # This is a do-nothing fall-back method
+        # Call the H5Fflush with this Leaf
+	hdf5Extension.flush_leaf(self._v_parent, self._v_hdf5name)
 
-    def close(self):
+    def close(self, flush=1):
         """Flush the buffers and close this object on tree"""
-        self.flush()
+        if flush:
+            self.flush()
         parent = self._v_parent
         del parent._v_leaves[self._v_name]
         del parent.__dict__[self._v_name]
@@ -428,12 +450,17 @@ class Leaf:
         if hasattr(self, "attrs"): 
             self.attrs._f_close()
             del self.attrs
+            del self.filters
 
         # After the objects are disconnected, destroy the
         # object dictionary using the brute force ;-)
         # This should help to the garbage collector
         #self.__dict__.clear()
 
+    def __len__(self):
+        "Useful for dealing with Leaf objects as sequences"
+        return self.nrows
+
     def __str__(self):
         """The string reprsentation choosed for this object is its pathname
         in the HDF5 object tree.
