@@ -1,3 +1,6 @@
+# Eh! python!, We are going to include isolatin characters here
+# -*- coding: latin-1 -*-
+
 ########################################################################
 #
 #       License: BSD
@@ -5,7 +8,7 @@
 #       Author:  Francesc Alted - falted@pytables.org
 #
 #       $Source: /cvsroot/pytables/pytables/tables/VLArray.py,v $
-#       $Id: VLArray.py,v 1.26 2004/02/25 16:08:59 falted Exp $
+#       $Id: VLArray.py,v 1.36.2.1 2004/11/25 11:50:37 falted Exp $
 #
 ########################################################################
 
@@ -30,10 +33,11 @@ Misc variables:
 
 """
 
-__version__ = "$Revision: 1.26 $"
+__version__ = "$Revision: 1.36.2.1 $"
 
 # default version for VLARRAY objects
-obversion = "1.0"    # initial version
+#obversion = "1.0"    # initial version
+obversion = "1.0"    # add support for complex datatypes
 
 import types, warnings, sys
 import cPickle
@@ -42,7 +46,8 @@ import numarray.strings as strings
 import numarray.records as records
 from Leaf import Leaf
 import hdf5Extension
-from IsDescription import Col, BoolCol, StringCol, IntCol, FloatCol
+#import IsDescription # to access BaseCol without polluting public namespace
+from IsDescription import Col, BoolCol, StringCol, IntCol, FloatCol, ComplexCol
 from utils import processRange, processRangeRead, convertIntoNA
 
 try:
@@ -101,6 +106,7 @@ class ObjectAtom(IntCol):
         return 1
 
 
+#class Atom(IsDescription.BaseCol):
 class Atom(Col):
     """ Define an Atomic object to be used in VLArray objects """
 
@@ -221,6 +227,24 @@ class Float64Atom(Atom, FloatCol):
         FloatCol.__init__(self, shape=shape, itemsize=8)
         self.flavor = checkflavor(flavor, self.type)
 
+class ComplexAtom(Atom, ComplexCol):
+    """ Define an atom of type Complex """
+    def __init__(self, shape=1, itemsize=16, flavor="NumArray"):
+        ComplexCol.__init__(self, shape=shape, itemsize=itemsize)
+        self.flavor = checkflavor(flavor, self.type)
+
+class Complex32Atom(Atom, ComplexCol):
+    """ Define an atom of type Complex32 """
+    def __init__(self, shape=1, flavor="NumArray"):
+        ComplexCol.__init__(self, shape=shape, itemsize=8)
+        self.flavor = checkflavor(flavor, self.type)
+
+class Complex64Atom(Atom, ComplexCol):
+    """ Define an atom of type Complex64 """
+    def __init__(self, shape=1, flavor="NumArray"):
+        ComplexCol.__init__(self, shape=shape, itemsize=16)
+        self.flavor = checkflavor(flavor, self.type)
+
         
 def calcChunkSize(expectedsizeinMB, complevel):
     """Computes the optimum value for the chunksize"""
@@ -268,6 +292,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         __iter__()
         iterrows(start, stop, step)
         __getitem__(slice)
+        __setitem__(slice, value)
 
     Instance variables:
 
@@ -380,6 +405,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
             
     def append(self, *objects):
         """Append the objects to this enlargeable object"""
+        assert not self._v_file.mode == "r", "Attempt to write over a file opened in read-only mode"
 
         # To make append([1,0,1]) equivalent to append(1,0,1)
         if len(objects) == 0:
@@ -445,16 +471,8 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
                                  self.flavor)
 
     def iterrows(self, start=None, stop=None, step=None):
-        """Iterator over all the rows or a range"""
-
-        return self.__call__(start, stop, step)
-
-    def __call__(self, start=None, stop=None, step=None):
         """Iterate over all the rows or a range.
-        
-        It returns the same iterator than
-        Table.iterrows(start, stop, step).
-        It is, therefore, a shorter way to call it.
+
         """
 
         (self._start, self._stop, self._step) = \
@@ -549,7 +567,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         return arr
 
     def __getitem__(self, key):
-        """Returns a table row, table slice or table column.
+        """Returns a vlarray row or slice.
 
         It takes different actions depending on the type of the "key"
         parameter:
@@ -557,44 +575,132 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         If "key"is an integer, the corresponding row is returned. If
         "key" is a slice, the row slice determined by key is returned.
 
-"""
+        """
 
         if isinstance(key, types.IntType):
-            (start, stop, step) = processRange(self.nrows, key, key+1, 1)
-            return self.read(start, stop, step, slice_specified=0)
+            if key >= self.nrows:
+                raise IndexError, "Index out of range"
+            if key < 0:
+                # To support negative values
+                key += self.nrows
+            return self.read(key)[0]
         elif isinstance(key, types.SliceType):
-            (start, stop, step) = processRange(self.nrows,
-                                               key.start, key.stop, key.step)
-            return self.read(start, stop, step, slice_specified=1)
+            return self.read(key.start, key.stop, key.step)
         else:
-            raise ValueError, "Non-valid index or slice: %s" % \
+            raise IndexError, "Non-valid index or slice: %s" % \
                   key
         
+    def __setitem__(self, keys, value):
+        """Updates a vlarray row "keys" by setting it to "value".
+
+        If "keys" is an integer, it refers to the number of row to be
+        modified.
+
+        If "keys" is a tuple, the first element refers to the row
+        to be modified, and the second element to the range (so, it
+        can be an integer or an slice) of the row that will be
+        updated.
+
+        Note: When updating VLStrings (codification UTF-8) or Objects,
+        there is a problem: we can only update values with *exactly*
+        the same bytes than in the original row. With UTF-8 encoding
+        this is problematic because, for instance, 'c' takes 1 byte,
+        but '' takes at least two (!). Perhaps another codification
+        does not have this problem, I don't know. With objects, the
+        same happens, because cPickle applied on an instance (for
+        example) does not guarantee to return the same number of bytes
+        than over other instance, even of the same class than the
+        former. This effectively limits the number of objects than can
+        be updated in VLArrays, most specially VLStrings and Objects
+        as has been said before.
+
+        """
+
+        assert not self._v_file.mode == "r", "Attempt to write over a file opened in read-only mode"
+
+        if not isinstance(keys, types.TupleType):
+            keys = (keys, None)
+        if len(keys) > 2:
+            raise IndexError, "You cannot specify more than two dimensions"
+        nrow, rng = keys
+        # Process the first index
+        if not isinstance(nrow, types.IntType):
+            raise IndexError, "The first dimension only can be an integer"
+        if nrow >= self.nrows:
+            raise IndexError, "First index out of range"
+        if nrow < 0:
+            # To support negative values
+            nrow += self.nrows
+        # Process the second index
+        if isinstance(rng, types.IntType):
+            start = rng; stop = start+1; step = 1
+        elif isinstance(rng, types.SliceType):
+            start, stop, step = rng.start, rng.stop, rng.step
+        elif isinstance(rng, types.NoneType):
+            start, stop, step = None, None, None
+        else:
+            raise IndexError, "Non-valid second index or slice: %s" % rng
+        
+        object = value
+        # Prepare the object to convert it into a numarray object
+        if self.atom.flavor == "Object":
+            # Special case for a generic object
+            # (to be pickled and saved as an array of unsigned bytes)
+            object = numarray.array(cPickle.dumps(object), type=numarray.UInt8)
+        elif self.atom.flavor == "VLString":
+            # Special case for a generic object
+            # (to be pickled and saved as an array of unsigned bytes)
+            if not (isinstance(object, types.StringType) or
+                    isinstance(object, types.UnicodeType)):
+                raise TypeError, \
+"""The object "%s" is not of type String or Unicode.""" % (str(object))
+            try:
+                object = object.encode('utf-8')
+            except:
+                (type, value, traceback) = sys.exc_info()
+                raise ValueError, "Problems when converting the object '%s' to the encoding 'utf-8'. The error was: %s" % (object, value)
+            object = numarray.array(object, type=numarray.UInt8)
+
+        value = convertIntoNA(object, self.atom)
+        nobjects = self._checkShape(value)
+
+        #nobjects = len(value)
+
+        # Get the previous value
+        #naarr = self.read(nrow, nrow+1, 1, slice_specified=0)
+        naarr = self._readArray(nrow, nrow+1, 1)[0]
+        nobjects = len(naarr)
+        if len(value) > nobjects:
+            raise ValueError, \
+"Length of value (%s) is larger than number of elements in row (%s)" % \
+(len(value), nobjects)
+        # Assign the value to it
+        try:
+            naarr[slice(start, stop, step)] = value
+        except:
+            (type, value2, traceback) = sys.exc_info()
+            raise ValueError, \
+"Value parameter:\n'%r'\ncannot be converted into an array object compliant vlarray[%s] row: \n'%r'\nThe error was: <%s>" % \
+        (value, keys, naarr[slice(start, stop, step)], value2)
+
+        if naarr.size() > 0:
+            self._modify(nrow, naarr, nobjects)
+
     # Accessor for the _readArray method in superclass
-    def read(self, start=None, stop=None, step=1, slice_specified=0):
+    def read(self, start=None, stop=None, step=1):
         """Read the array from disk and return it as a self.flavor object."""
 
-#         if stop <> None:
-#             stop_specified = 1
         start, stop, step = processRangeRead(self.nrows, start, stop, step)
-
         if start == stop:
             listarr = []
         else:
             listarr = self._readArray(start, stop, step)
-
         if self.flavor <> "NumArray":
             # Convert the list to the right flavor
             outlistarr = [self._convToFlavor(arr) for arr in listarr ]
         else:
             # NumArray flavor does not need additional conversion
             outlistarr = listarr
-
-        # Check for unitary length lists or tuples
-        #if len(outlistarr) == 1 and not stop_specified:
-        if len(outlistarr) == 1 and not slice_specified:
-            outlistarr = outlistarr[0]
-            
         return outlistarr
 
     def _g_copy(self, group, name, start, stop, step, title, filters):
@@ -615,7 +721,7 @@ class VLArray(Leaf, hdf5Extension.VLArray, object):
         nrowscopied = 0
         nbytes = 0
         atomsize = self.atom.atomsize()
-        for start2 in range(start, stop, step*nrowsinbuf):
+        for start2 in xrange(start, stop, step*nrowsinbuf):
             # Save the records on disk
             stop2 = start2+step*nrowsinbuf
             if stop2 > stop:
