@@ -16,7 +16,7 @@ from tables import *
 # important objects to test
 from tables import File, Group, Leaf, Table, Array
 
-from test_all import verbose
+from test_all import verbose, heavy
 
 class Record(IsDescription):
     var1 = StringCol(length=4)     # 4-character String
@@ -154,10 +154,10 @@ class createTestCase(unittest.TestCase):
 	assert arr.getAttr("TITLE") == "t" * titlelength
 
     def test04_maxFields(self):
-	"Checking the maximum number of fields (255) in tables"
+	"Checking the maximum number of fields (256) in tables"
 
 	# The number of fields for a table
-	varnumber = 255
+	varnumber = 256
 
 	varnames = []
 	for i in range(varnumber):
@@ -167,43 +167,42 @@ class createTestCase(unittest.TestCase):
 	recordDict = {}
 	i = 0
 	for varname in varnames:
-	    recordDict[varname] = Col("Int32", 1)
+	    recordDict[varname] = Col("Int32", 1, pos=i)
 	    i += 1
 	# Append this entry to indicate the alignment!
 	recordDict['_v_align'] = "="
-	# Now, create a table with this record object
-	table = Table(recordDict, "MetaRecord instance")
-	# Attach the table to the object tree
-	self.root.table = table
-        # This works the same than above
-	#table = self.fileh.createTable(self.root, 'table',
-        #                               recordDict, "MetaRecord instance")
+	table = self.fileh.createTable(self.root, 'table',
+                                       recordDict, "MetaRecord instance")
 	row = table.row
+        listrows = []
 	# Write 10 records
         for j in range(10):
-            i = 0
-            for varname in varnames:
-                row[varname] = i*j
-                i += 1
+            rowlist = []
+            for i in range(len(table.cols)):
+                row[varnames[i]] = i*j
+                rowlist.append(i*j)
 	    
             row.append()
+            listrows.append(tuple(rowlist))
 
         # write data on disk
 	table.flush()
 
-	# Read all the data as records 
-	for recout in table.iterrows():
-            pass
+	# Read all the data as a list
+	listout = table.read(flavor="List")
 
-        # Compare the last input row and last output
-        # they should be equal
-        assert row == recout
+        # Compare the input rowlist and output row list. They should
+        # be equal.
+        if verbose:
+            print "Original row list:", listrows[-1]
+            print "Retrieved row list:", listout[-1]
+        assert listrows == listout
 
     def test05_maxFieldsExceeded(self):
-	"Checking an excess (256) of the maximum number of fields in tables"
+	"Checking an excess (257) of the maximum number of fields in tables"
 
 	# The number of fields for a table
-	varnumber = 256
+	varnumber = 257
 
 	varnames = []
 	for i in range(varnumber):
@@ -221,12 +220,15 @@ class createTestCase(unittest.TestCase):
 	recordDict['_v_align'] = "="
         
 	# Now, create a table with this record object
-	table = Table(recordDict, "MetaRecord instance")
+        # This way of creating node objects has been deprecated
+	#table = Table(recordDict, "MetaRecord instance")
 
 	# Attach the table to object tree
         # Here, IndexError should be raised!
         try:
-            self.root.table = table
+            #self.root.table = table
+            table = self.fileh.createTable(self.root, 'table',
+                                           recordDict, "MetaRecord instance")
         except IndexError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
@@ -244,12 +246,15 @@ class createTestCase(unittest.TestCase):
 	recordDict["b"*256] = IntCol(1) # Should trigger a NameError
         
 	# Now, create a table with this record object
+        # This way of creating node objects has been deprecated
 	table = Table(recordDict, "MetaRecord instance")
 
 	# Attach the table to object tree
         # Here, IndexError should be raised!
         try:
-            self.root.table = table
+            # self.root.table = table
+            table = self.fileh.createTable(self.root, 'table',
+                                           recordDict, "MetaRecord instance")
         except NameError:
             if verbose:
                 (type, value, traceback) = sys.exc_info()
@@ -461,6 +466,47 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.agroup._v_attrs._f_list("all") == \
                ['CLASS', 'FILTERS', 'TITLE', 'VERSION', "rs"]
 
+    def test05b_removeAttributes(self):
+        """Checking removing attributes (using File.delAttrNode()) """
+
+        # With a Group object
+        self.group._v_attrs.pq = "1"
+        self.group._v_attrs.qr = "2"
+        self.group._v_attrs.rs = "3"
+        # delete an attribute
+        self.fileh.delAttrNode(self.group, "pq")
+        
+        if self.close:
+            if verbose:
+                print "(closing file version)"
+            self.fileh.close()
+            self.fileh = openFile(self.file, mode = "r+")
+            self.root = self.fileh.root
+            
+        if verbose:
+            print "Attribute list:", self.root.agroup._v_attrs._f_list()
+        # Check the local attributes names
+        assert self.root.agroup._v_attrs._f_list() == ["qr", "rs"]
+        if verbose:
+            print "Attribute list in disk:", \
+                  self.root.agroup._v_attrs._f_list("all")
+        # Check the disk attribute names
+        assert self.root.agroup._v_attrs._f_list("all") == \
+               ['CLASS', 'FILTERS', 'TITLE', 'VERSION', "qr", "rs"]
+
+        # delete an attribute (File.delAttrNode method)
+        self.fileh.delAttrNode(self.root, "qr", "agroup")
+        if verbose:
+            print "Attribute list:", self.root.agroup._v_attrs._f_list()
+        # Check the local attributes names
+        assert self.root.agroup._v_attrs._f_list() == ["rs"]
+        if verbose:
+            print "Attribute list in disk:", \
+                  self.root.agroup._v_attrs._g_listAttr()
+        # Check the disk attribute names
+        assert self.root.agroup._v_attrs._f_list("all") == \
+               ['CLASS', 'FILTERS', 'TITLE', 'VERSION', "rs"]
+
     def test06_removeAttributes(self):
         """Checking removing system attributes """
 
@@ -543,7 +589,9 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.atable.attrs.qr == 2
         assert self.root.atable.attrs.rs == 3
 
-    def test09b_setIntAttributes(self):
+    # This test is commented out, because the necessary API is not
+    # public
+    def _test09b_setIntAttributes(self):
         """Checking setting Int (byte, short, int) attributes"""
 
         # With a Table object
@@ -601,7 +649,9 @@ class createAttrTestCase(unittest.TestCase):
         assert self.root.atable.attrs.qr == 2.0
         assert self.root.atable.attrs.rs == 3.0
 
-    def test10b_setFloatAttributes(self):
+    # This test is commented out, because the necessary API is not
+    # public
+    def _test10b_setFloatAttributes(self):
         """Checking setting Float (float) attributes"""
 
         # With a Table object
@@ -1095,8 +1145,7 @@ class CopyGroupTestCase(unittest.TestCase):
 #                                recursive=0,
 #                                filters=self.filters)
         self.h5file.copyChildren(srcgroup, self.h5file2.root,
-                               recursive=0,
-                               filters=self.filters)
+                                 recursive=0, filters=self.filters)
         if self.close:
             # Close the destination file
             self.h5file2.close()
@@ -1125,9 +1174,9 @@ class CopyGroupTestCase(unittest.TestCase):
         # Copy a group non-recursively with attrs
         srcgroup = self.h5file.root.group0.group1
         srcgroup._f_copyChildren(self.h5file2.root,
-                               recursive=0,
-                               filters=self.filters,
-                               copyuserattrs = 1)
+                                 recursive=0,
+                                 filters=self.filters,
+                                 copyuserattrs = 1)
         if self.close:
             # Close the destination file
             self.h5file2.close()
@@ -1213,7 +1262,7 @@ class CopyGroupTestCase(unittest.TestCase):
             lenDstGroup = 0  # Case where dstgroup == "/"
         first = 1
         nodelist1 = []
-        for node in srcgroup(recursive=1):
+        for node in srcgroup._f_walkNodes(recursive=1):
             if first:
                 # skip the first group
                 first = 0
@@ -1222,7 +1271,7 @@ class CopyGroupTestCase(unittest.TestCase):
 
         first = 1
         nodelist2 = []
-        for node in dstgroup(recursive=1):
+        for node in dstgroup._f_walkNodes(recursive=1):
             if first:
                 # skip the first group
                 first = 0
@@ -1269,7 +1318,7 @@ class CopyGroupTestCase(unittest.TestCase):
             lenDstGroup = 0  # Case where dstgroup == "/"
         first = 1
         nodelist1 = {}
-        for node in srcgroup(recursive=1):
+        for node in srcgroup._f_walkNodes(recursive=1):
             if first:
                 # skip the first group
                 first = 0
@@ -1277,7 +1326,7 @@ class CopyGroupTestCase(unittest.TestCase):
             nodelist1[node._v_name] = node._v_pathname[lenSrcGroup:]
 
         first = 1
-        for node in dstgroup(recursive=1):
+        for node in dstgroup._f_walkNodes(recursive=1):
             if first:
                 # skip the first group
                 first = 0
@@ -1713,7 +1762,9 @@ class CopyFileCase10(unittest.TestCase):
 def suite():
     theSuite = unittest.TestSuite()
     niter = 1
-
+    #heavy = 1 # Uncomment this only for testing purposes!
+    
+    #theSuite.addTest(unittest.makeSuite(createTestCase))
     #theSuite.addTest(unittest.makeSuite(CopyGroupCase1))
     #theSuite.addTest(unittest.makeSuite(CopyGroupCase2))
     for i in range(niter):
@@ -1722,30 +1773,30 @@ def suite():
         theSuite.addTest(unittest.makeSuite(createAttrCloseTestCase))
         theSuite.addTest(unittest.makeSuite(FiltersCase1))
         theSuite.addTest(unittest.makeSuite(FiltersCase2))
+        theSuite.addTest(unittest.makeSuite(CopyGroupCase1))
+        theSuite.addTest(unittest.makeSuite(CopyGroupCase2))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase1))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase2))
+    if heavy:
         theSuite.addTest(unittest.makeSuite(FiltersCase3))
         theSuite.addTest(unittest.makeSuite(FiltersCase4))
         theSuite.addTest(unittest.makeSuite(FiltersCase5))
         theSuite.addTest(unittest.makeSuite(FiltersCase6))
         theSuite.addTest(unittest.makeSuite(FiltersCase7))
         theSuite.addTest(unittest.makeSuite(FiltersCase8))
-        theSuite.addTest(unittest.makeSuite(CopyGroupCase1))
-        theSuite.addTest(unittest.makeSuite(CopyGroupCase2))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase3))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase4))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase5))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase6))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase7))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase8))
+        theSuite.addTest(unittest.makeSuite(CopyFileCase10))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase3))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase4))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase5))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase6))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase7))
         theSuite.addTest(unittest.makeSuite(CopyGroupCase8))
-        theSuite.addTest(unittest.makeSuite(CopyFileCase1))
-        theSuite.addTest(unittest.makeSuite(CopyFileCase2))
-        theSuite.addTest(unittest.makeSuite(CopyFileCase3))
-        theSuite.addTest(unittest.makeSuite(CopyFileCase4))
-        # These take too much time and does not apport nothing really new
-#         theSuite.addTest(unittest.makeSuite(CopyFileCase5))
-#         theSuite.addTest(unittest.makeSuite(CopyFileCase6))
-#         theSuite.addTest(unittest.makeSuite(CopyFileCase7))
-#         theSuite.addTest(unittest.makeSuite(CopyFileCase8))
-        theSuite.addTest(unittest.makeSuite(CopyFileCase10))
 
     return theSuite
 
