@@ -3,6 +3,7 @@ import unittest
 import os
 import tempfile
 import warnings
+import types
 import numarray
 from numarray import *
 import numarray.strings as strings
@@ -16,9 +17,7 @@ from tables import *
 
 from test_all import verbose, allequal
 
-# If the next line is uncommented, the logical_and.reduce in loop issues an
-# OverflowWarning in the case of type Int8 (and only in that case).
-#warnings.resetwarnings()
+warnings.resetwarnings()
 
 class BasicTestCase(unittest.TestCase):
     """Basic test for all the supported typecodes present in numarray.
@@ -90,7 +89,6 @@ class BasicTestCase(unittest.TestCase):
 
         assert allequal(a,b)
 
-        # Close the file (eventually destroy the extended type)
         self.fileh.close()
 
         # Then, delete the file
@@ -120,7 +118,7 @@ class BasicTestCase(unittest.TestCase):
 
         # UInt64 seems to be unsupported on 64-bit machines!
 	typecodes = [Int8, UInt8, Int16, UInt16, Int32, UInt32, Int64,
-                     Float32, Float64]
+                     Float32, Float64, Complex32, Complex64]
 
 	for typecode in typecodes:
             a = array(self.tupleInt, typecode)
@@ -133,7 +131,7 @@ class BasicTestCase(unittest.TestCase):
 
         # UInt64 seems to be unsupported on 64-bit machines!
 	typecodes = [Int8, UInt8, Int16, UInt16, Int32, UInt32, Int64,
-                     Float32, Float64]
+                     Float32, Float64, Complex32, Complex64]
 
 	for typecode in typecodes:
             a = array(self.tupleInt, typecode)
@@ -227,7 +225,6 @@ class UnalignedAndComplexTestCase(unittest.TestCase):
         self.root = self.fileh.root
 
     def tearDown(self):
-        # Close the file (eventually destroy the extended type)
         self.fileh.close()
 
         # Then, delete the file
@@ -390,33 +387,6 @@ class UnalignedAndComplexTestCase(unittest.TestCase):
         assert a.type() == numarray.Float64
 	self.WriteRead(a)
 	return
-    
-    def test11_complexSimple(self):
-        "Checking a complex floating point array (not supported)"
-	a = numarray.array( [1,2], numarray.Complex32)
-        try:
-            self.WriteRead(a)
-        except TypeError:
-            if verbose:
-                (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next TypeError was catched!"
-                print value
-        else:
-            self.fail("expected an TypeError")
-            
-    def test12_complexDouble(self):
-        "Checking a complex floating point array (not supported)"
-
-	a = numarray.array( [1,2], numarray.Complex64)
-        try:
-            self.WriteRead(a)
-        except TypeError:
-            if verbose:
-                (type, value, traceback) = sys.exc_info()
-                print "\nGreat!, the next TypeError was catched!"
-                print value
-        else:
-            self.fail("expected an TypeError")
 
     
 class GroupsArrayTestCase(unittest.TestCase):
@@ -446,7 +416,7 @@ class GroupsArrayTestCase(unittest.TestCase):
 	#typecodes = ["c", 'b', '1', 's', 'w', 'i', 'u', 'l', 'f', 'd']
         # UInt64 seems to be unsupported on 64-bit machines!
 	typecodes = [Int8, UInt8, Int16, UInt16, Int32, UInt32, Int64,
-                     Float32, Float64]
+                     Float32, Float64, Complex32, Complex64]
 	i = 1
 	for typecode in typecodes:
 	    # Create an array of typecode, with incrementally bigger ranges
@@ -829,8 +799,8 @@ class CopyTestCase(unittest.TestCase):
             print "attrs array2-->", repr(array2.attrs)
             
         # Assert user attributes
-        array2.attrs.attr1 == None
-        array2.attrs.attr2 == None
+        hasattr(array2.attrs, "attr1") == 0
+        hasattr(array2.attrs, "attr2") == 0
 
         # Close the file
         fileh.close()
@@ -993,13 +963,689 @@ class CopyIndex12TestCase(CopyIndexTestCase):
     stop = None  # None should mean the last element (including it)
     step = 1
 
+class GetItemTestCase(unittest.TestCase):
+
+    def test00_single(self):
+        "Single element access (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original first element:", a[0]
+            print "Read first element:", arr[0]
+        assert allequal(a[0], arr[0])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test01_single(self):
+        "Single element access (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original first element:", a[0]
+            print "Read first element:", arr[0]
+        assert a[0] == arr[0]
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test02_range(self):
+        "Range element access (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4]
+            print "Read elements:", arr[1:4]
+        assert allequal(a[1:4], arr[1:4])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test03_range(self):
+        "Range element access (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4]
+            print "Read elements:", arr[1:4]
+        assert allequal(a[1:4], arr[1:4])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test04_range(self):
+        "Range element access, strided (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4:2]
+            print "Read elements:", arr[1:4:2]
+        assert allequal(a[1:4:2], arr[1:4:2])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test05_range(self):
+        "Range element access, strided (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4:2]
+            print "Read elements:", arr[1:4:2]
+        assert allequal(a[1:4:2], arr[1:4:2])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test06_negativeIndex(self):
+        "Negative Index element access (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original last element:", a[-1]
+            print "Read last element:", arr[-1]
+        assert allequal(a[-1], arr[-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test07_negativeIndex(self):
+        "Negative Index element access (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original before last element:", a[-2]
+            print "Read before last element:", arr[-2]
+        if isinstance(a[-2], numarray.NumArray):
+            assert allequal(a[-2], arr[-2])
+        else:
+            assert a[-2] == arr[-2]
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test08_negativeRange(self):
+        "Negative range element access (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original last elements:", a[-4:-1]
+            print "Read last elements:", arr[-4:-1]
+        assert allequal(a[-4:-1], arr[-4:-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test09_negativeRange(self):
+        "Negative range element access (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        if verbose:
+            print "Original last elements:", a[-4:-1]
+            print "Read last elements:", arr[-4:-1]
+        assert allequal(a[-4:-1], arr[-4:-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+
+class GI1NATestCase(GetItemTestCase):
+    title = "Rank-1 case 1"
+    numericalList = numarray.array([3])
+    numericalListME = numarray.array([3,2,1,0,4,5,6])
+    charList = strings.array(["3"])
+    charListME = strings.array(["321","221","121","021","421","521","621"])
+    
+class GI2NATestCase(GetItemTestCase):
+    # A more complex example
+    title = "Rank-1,2 case 2"
+    numericalList = numarray.array([3,4])
+    numericalListME = numarray.array([[3,2,1,0,4,5,6],
+                                      [2,1,0,4,5,6,7],
+                                      [4,3,2,1,0,4,5],
+                                      [3,2,1,0,4,5,6],
+                                      [3,2,1,0,4,5,6]])
+    
+    charList = strings.array(["a","b"])
+    charListME = strings.array([["321","221","121","021","421","521","621"],
+                                ["21","21","11","02","42","21","61"],
+                                ["31","21","12","21","41","51","621"],
+                                ["321","221","121","021","421","521","621"],
+                                ["3241","2321","13216","0621","4421","5421","a621"],
+                                ["a321","s221","d121","g021","b421","5vvv21","6zxzxs21"]])
+    
+
+class SetItemTestCase(unittest.TestCase):
+
+    def test00_single(self):
+        "Single element update (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify a single element of a and arr:
+        a[0] = "b"
+        arr[0] = "b"
+
+        # Get and compare an element
+        if verbose:
+            print "Original first element:", a[0]
+            print "Read first element:", arr[0]
+        assert allequal(a[0], arr[0])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test01_single(self):
+        "Single element update (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        a[0] = 333
+        arr[0] = 333
+
+        # Get and compare an element
+        if verbose:
+            print "Original first element:", a[0]
+            print "Read first element:", arr[0]
+        assert a[0] == arr[0]
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test02_range(self):
+        "Range element update (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        a[1:3] = "xXx"
+        arr[1:3] = "xXx"
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4]
+            print "Read elements:", arr[1:4]
+        assert allequal(a[1:4], arr[1:4])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test03_range(self):
+        "Range element update (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = slice(1,3,None)
+        a[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+        arr[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4]
+            print "Read elements:", arr[1:4]
+        assert allequal(a[1:4], arr[1:4])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test04_range(self):
+        "Range element update, strided (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = slice(1,4,2)
+        a[s] = "xXx"
+        arr[s] = "xXx"
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4:2]
+            print "Read elements:", arr[1:4:2]
+        assert allequal(a[1:4:2], arr[1:4:2])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test05_range(self):
+        "Range element update, strided (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = slice(1,4,2)
+        a[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+        arr[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+
+        # Get and compare an element
+        if verbose:
+            print "Original elements:", a[1:4:2]
+            print "Read elements:", arr[1:4:2]
+        assert allequal(a[1:4:2], arr[1:4:2])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test06_negativeIndex(self):
+        "Negative Index element update (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = -1
+        a[s] = "xXx"
+        arr[s] = "xXx"
+
+        # Get and compare an element
+        if verbose:
+            print "Original last element:", a[-1]
+            print "Read last element:", arr[-1]
+        assert allequal(a[-1], arr[-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test07_negativeIndex(self):
+        "Negative Index element update (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = -2
+        a[s] = a[s]*2+3
+        arr[s] = arr[s]*2+3
+
+        # Get and compare an element
+        if verbose:
+            print "Original before last element:", a[-2]
+            print "Read before last element:", arr[-2]
+        if isinstance(a[-2], numarray.NumArray):
+            assert allequal(a[-2], arr[-2])
+        else:
+            assert a[-2] == arr[-2]
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test08_negativeRange(self):
+        "Negative range element update (character types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = slice(-4,-1,None)
+        a[s] = "xXx"
+        arr[s] = "xXx"
+
+        # Get and compare an element
+        if verbose:
+            print "Original last elements:", a[-4:-1]
+            print "Read last elements:", arr[-4:-1]
+        assert allequal(a[-4:-1], arr[-4:-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test09_negativeRange(self):
+        "Negative range element update (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of a and arr:
+        s = slice(-3,-1,None)
+        a[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+        arr[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+
+        # Get and compare an element
+        if verbose:
+            print "Original last elements:", a[-4:-1]
+            print "Read last elements:", arr[-4:-1]
+        assert allequal(a[-4:-1], arr[-4:-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test10_outOfRange(self):
+        "Out of range update (numerical types)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Modify elements of arr that are out of range:
+        s = slice(1,a.shape[0]+1,None)
+        s2 = slice(1,1000,None)
+        a[s] = arange(a[s].size(),shape=a[s].shape)*2+3
+        arr[s2] = arange(a[s2].size(),shape=a[s2].shape)*2+3
+
+        # Get and compare an element
+        if verbose:
+            print "Original last elements:", a[-4:-1]
+            print "Read last elements:", arr[-4:-1]
+        assert allequal(a[-4:-1], arr[-4:-1])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+class SI1NATestCase(SetItemTestCase):
+    title = "Rank-1 case 1"
+    numericalList = numarray.array([3])
+    numericalListME = numarray.array([3,2,1,0,4,5,6])
+    charList = strings.array(["3"])
+    charListME = strings.array(["321","221","121","021","421","521","621"])
+    
+class SI2NATestCase(SetItemTestCase):
+    # A more complex example
+    title = "Rank-1,2 case 2"
+    numericalList = numarray.array([3,4])
+    numericalListME = numarray.array([[3,2,1,0,4,5,6],
+                                      [2,1,0,4,5,6,7],
+                                      [4,3,2,1,0,4,5],
+                                      [3,2,1,0,4,5,6],
+                                      [3,2,1,0,4,5,6]])
+    
+    charList = strings.array(["a","b"])
+    charListME = strings.array([["321","221","121","021","421","521","621"],
+                                ["21","21","11","02","42","21","61"],
+                                ["31","21","12","21","41","51","621"],
+                                ["321","221","121","021","421","521","621"],
+                                ["3241","2321","13216","0621","4421","5421","a621"],
+                                ["a321","s221","d121","g021","b421","5vvv21","6zxzxs21"]])
+    
+
+class GeneratorTestCase(unittest.TestCase):
+
+    def test00a_single(self):
+        "Testing generator access to Arrays, single elements (char)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        ga = [i for i in a]
+        garr = [i for i in arr]
+        if verbose:
+            print "Result of original iterator:", ga
+            print "Result of read generator:", garr
+        assert ga == garr
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test00b_me(self):
+        "Testing generator access to Arrays, multiple elements (char)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.charListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        ga = [i for i in a]
+        garr = [i for i in arr]
+        if verbose:
+            print "Result of original iterator:", ga
+            print "Result of read generator:", garr
+        for i in range(len(ga)):
+            assert allequal(ga[i], garr[i])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test01a_single(self):
+        "Testing generator access to Arrays, single elements (numeric)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalList
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        ga = [i for i in a]
+        garr = [i for i in arr]
+        if verbose:
+            print "Result of original iterator:", ga
+            print "Result of read generator:", garr
+        assert ga == garr
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+    def test01b_me(self):
+        "Testing generator access to Arrays, multiple elements (numeric)"
+        
+        file = tempfile.mktemp(".h5")
+        fileh = openFile(file, mode = "w")
+        # Create the array under root and name 'somearray'
+        a = self.numericalListME
+        arr=fileh.createArray(fileh.root, 'somearray', a, "Some array")
+
+        # Get and compare an element
+        ga = [i for i in a]
+        garr = [i for i in arr]
+        if verbose:
+            print "Result of original iterator:", ga
+            print "Result of read generator:", garr
+        for i in range(len(ga)):
+            assert allequal(ga[i], garr[i])
+        
+        # Close the file
+        fileh.close()
+        # Then, delete the file
+        os.remove(file)
+        return
+
+class GE1NATestCase(GeneratorTestCase):
+    title = "Rank-1 case 1"
+    numericalList = numarray.array([3])
+    numericalListME = numarray.array([3,2,1,0,4,5,6])
+    charList = strings.array(["3"])
+    charListME = strings.array(["321","221","121","021","421","521","621"])
+    
+class GE2NATestCase(GeneratorTestCase):
+    # A more complex example
+    title = "Rank-1,2 case 2"
+    numericalList = numarray.array([3,4])
+    numericalListME = numarray.array([[3,2,1,0,4,5,6],
+                                      [2,1,0,4,5,6,7],
+                                      [4,3,2,1,0,4,5],
+                                      [3,2,1,0,4,5,6],
+                                      [3,2,1,0,4,5,6]])
+    
+    charList = strings.array(["a","b"])
+    charListME = strings.array([["321","221","121","021","421","521","621"],
+                                ["21","21","11","02","42","21","61"],
+                                ["31","21","12","21","41","51","621"],
+                                ["321","221","121","021","421","521","621"],
+                                ["3241","2321","13216","0621","4421","5421","a621"],
+                                ["a321","s221","d121","g021","b421","5vvv21","6zxzxs21"]])
+    
+
 def suite():
     theSuite = unittest.TestSuite()
     niter = 1
 
-    #theSuite.addTest(unittest.makeSuite(Basic2DTestCase))
-    #theSuite.addTest(unittest.makeSuite(CopyIndex12TestCase))
-    
     for i in range(niter):
         # The scalar case test should be refined in order to work
         theSuite.addTest(unittest.makeSuite(Basic0DOneTestCase))
@@ -1028,6 +1674,12 @@ def suite():
         theSuite.addTest(unittest.makeSuite(CopyIndex10TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex11TestCase))
         theSuite.addTest(unittest.makeSuite(CopyIndex12TestCase))
+        theSuite.addTest(unittest.makeSuite(GI1NATestCase))
+        theSuite.addTest(unittest.makeSuite(GI2NATestCase))
+        theSuite.addTest(unittest.makeSuite(SI1NATestCase))
+        theSuite.addTest(unittest.makeSuite(SI2NATestCase))
+        theSuite.addTest(unittest.makeSuite(GE1NATestCase))
+        theSuite.addTest(unittest.makeSuite(GE2NATestCase))
 
 
     return theSuite
