@@ -8,7 +8,9 @@ from tables import *
 # Next imports are only necessary for this test suite
 from tables import Group, Leaf, Table, Array
 
-from test_all import verbose
+from test_all import verbose, heavy
+
+#heavy = 1  # Uncomment this only for test purposes!
 
 # Test Record class
 class Record(IsDescription):
@@ -30,7 +32,6 @@ class TreeTestCase(unittest.TestCase):
         # Create an instance of HDF5 Table
         self.h5file = openFile(self.file, self.mode, self.title)
         self.populateFile()
-        # Close the file (eventually destroy the extended type)
         self.h5file.close()
             
     def populateFile(self):
@@ -336,28 +337,28 @@ class TreeTestCase(unittest.TestCase):
         if verbose:
             print "walkGroups(pathname) test passed"
 
-    def test04_iterTree(self):
-        "Checking the iterTree, iterGroup and family"
+    def test04_walkNodes(self):
+        "Checking File.walkNodes"
         
         if verbose:
             print '\n', '-=' * 30
-            print "Running %s.test04_iterTree..." % self.__class__.__name__
+            print "Running %s.test04_walkNodes..." % self.__class__.__name__
 
         self.h5file = openFile(self.file, "r")
         groups = []
         tables = []
         tables2 = []
         arrays = []
-        for group in self.h5file(classname="Group"):
+        for group in self.h5file.walkNodes(classname="Group"):
             groups.append(group._v_pathname)
-            for table in group(classname='Table'):
+            for table in group._f_walkNodes(classname='Table'):
                 tables.append(table._v_pathname)
                 
         # Test the recursivity    
-        for table in self.h5file.root(classname='Table', recursive=1):
+        for table in self.h5file.root._f_walkNodes('Table', recursive=1):
             tables2.append(table._v_pathname)
 
-        for arr in self.h5file(classname='Array'):
+        for arr in self.h5file.walkNodes(classname='Array'):
             arrays.append(arr._v_pathname)
 
         assert groups == ["/", "/group0", "/group0/group1",
@@ -376,11 +377,11 @@ class TreeTestCase(unittest.TestCase):
         groups = []
         tables = []
         arrays = []
-        for group in self.h5file("/group0/group1", classname="Group"):
+        for group in self.h5file.walkNodes("/group0/group1", classname="Group"):
             groups.append(group._v_pathname)
-            for table in group('Table'):
+            for table in group._f_walkNodes('Table'):
                 tables.append(table._v_pathname)
-            for arr in self.h5file(group, 'Array'):
+            for arr in self.h5file.walkNodes(group, 'Array'):
                 arrays.append(arr._v_pathname)
 
         assert groups == ["/group0/group1",
@@ -391,7 +392,7 @@ class TreeTestCase(unittest.TestCase):
         assert arrays == ['/group0/group1/var1', '/group0/group1/var4']
         
         if verbose:
-            print "iterTree(pathname, classname) test passed"
+            print "walkNodes(pathname, classname) test passed"
 
 
 class DeepTreeTestCase(unittest.TestCase):
@@ -403,16 +404,16 @@ class DeepTreeTestCase(unittest.TestCase):
     
     """
     def test00_deepTree(self):
-        
-        """Checking creation of large depth (512) object tree
-        Variable 'maxdepth' controls this check.
-        """
+        "Checking creation of large depth object tree Variable"
         
         # Here we put a more conservative limit to deal with more platforms
         # With maxdepth = 512 this test would take less than 20 MB
         # of main memory to run, which is quite reasonable nowadays.
         # With maxdepth = 1024 this test will take over 40 MB.
-        maxdepth = 512  # This should be safe for most machines
+        if heavy:
+            maxdepth = 1024  # Only for big machines!
+        else:
+            maxdepth = 256  # This should be safe for most machines
         
         if verbose:
             print '\n', '-=' * 30
@@ -487,7 +488,10 @@ class WideTreeTestCase(unittest.TestCase):
         """
 
         import time
-        maxchildren = 1024
+        if heavy:
+            maxchildren = 4096
+        else:
+            maxchildren = 256            
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test00_wideTree..." % \
@@ -548,7 +552,12 @@ class WideTreeTestCase(unittest.TestCase):
         """
 
         import time
-        maxchildren = 1024
+        if heavy:
+            # for big platforms!
+            maxchildren = 4096
+        else:
+            # for standard platforms
+            maxchildren = 256
         if verbose:
             print '\n', '-=' * 30
             print "Running %s.test00_wideTree..." % \
@@ -599,6 +608,7 @@ def suite():
     theSuite = unittest.TestSuite()
     # This counter is useful when detecting memory leaks
     niter = 1
+    #heavy=1  # see at the beginning
 
     #theSuite.addTest(unittest.makeSuite(DeepTreeTestCase))
     
